Package: leadeR
Title: Profiling Leaders at a Distance
Version: 0.1.0
Authors@R: c(
    person(given = "Mitsuru", family = "Mukaigawara", 
           email = "mitsuru_mukaigawara@g.harvard.edu", 
           role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-6530-2083")),
    person(given = "Joshua", family = "Kertzer", 
           email = "jkertzer@gov.harvard.edu", 
           role = "aut",
           comment = c(ORCID = "0000-0002-7358-0638")))
Description: Profiles political leaders at a distance from text data
    such as speeches, interviews, press conferences, and other public
    statements. Computes Leadership Trait Analysis scores for seven
    personality traits -- including need for power, conceptual complexity,
    and self-confidence -- and classifies leaders into one of eight
    leadership styles. Also computes Operational Code Analysis scores
    summarising a leader's beliefs about politics and the use of power.
URL: https://github.com/mmukaigawara/leadeR
License: MIT + file LICENSE
LazyData: true
Encoding: UTF-8
Depends: R (>= 4.1.0)
Imports: spacyr, dplyr, stringr, tidyr, stringi, data.table, tibble,
        countries, countrycode, purrr
Suggests: knitr, metafor, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-13 00:09:01 UTC; mitsurumukaigawara
Author: Mitsuru Mukaigawara [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-6530-2083>),
  Joshua Kertzer [aut] (ORCID: <https://orcid.org/0000-0002-7358-0638>)
Maintainer: Mitsuru Mukaigawara <mitsuru_mukaigawara@g.harvard.edu>
Repository: CRAN
Date/Publication: 2026-05-18 18:20:02 UTC
Built: R 4.6.0; ; 2026-05-18 20:17:14 UTC; unix

# Generated by roxygen2: do not edit by hand

export(clean_text)
export(get_aff)
export(get_complex)
export(get_conf)
export(get_ctrl)
export(get_dist)
export(get_lta)
export(get_nat)
export(get_oca)
export(get_power)
export(get_task)
export(type_lta)
import(dplyr)
import(stringr)
import(tidyr)
importFrom(countries,list_countries)
importFrom(countrycode,countryname)
importFrom(data.table,rleid)
importFrom(purrr,map_dfr)
importFrom(spacyr,spacy_parse)
importFrom(stats,na.omit)
importFrom(stats,var)
importFrom(stringi,stri_trans_general)
importFrom(tibble,tibble)

# Self-confidence

pronouns <- c("i", "me", "my", "mine", "myself")

#' Compute self-confidence scores
#'
#' Classifies first-person pronoun occurrences in the text as self-confident
#' (SC) or other-self-confident (OSC) based on sentence-level context patterns
#' covering three conditions: self as instigator, self as authority, and self
#' as recipient of positive recognition.
#'
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{SC} and \code{OSC}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanSC}, \code{meanOSC}, \code{varSC}, \code{varOSC}.
#' @export
get_conf <- function(text, bootstrap = FALSE, B = 1000) {

  # Parse text to tokens with spaCy
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)

  # Identify quoted spans to skip pronouns inside quotes
  parsed <- parsed |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(
      is_quote = FALSE,
      quote_open = token %in% c('"', "'", "\u201C", "\u201D", "\u2018", "\u2019"),
      quote_cumsum = cumsum(quote_open)) |>
    dplyr::mutate(in_quote = (quote_cumsum %% 2 == 1)) |>
    dplyr::ungroup()

  # Reconstruct sentences
  sentences <- parsed |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::summarise(sentence = paste(token, collapse = " "), .groups = "drop")

  # Filter for first-person pronouns outside quotes
  pronoun_df <- parsed |>
    dplyr::filter(tolower(token) %in% pronouns, !in_quote) |>
    dplyr::left_join(sentences, by = c("doc_id", "sentence_id"))

  if (nrow(pronoun_df) == 0) {
    if (bootstrap) {
      return(tibble::tibble(meanSC = 0, meanOSC = 0, varSC = 0, varOSC = 0))
    } else {
      return(tibble::tibble(SC = 0, OSC = 0))
    }
  } else {

    classify_conf <- function(sentence) {
      s <- tolower(sentence)

      # Condition A: Self as instigator of activity
      a_patterns <- c(
        "\\bi (will|shall|am going to|plan to|intend to|propose to|am prepared to|am ready to|am determined to|have decided to)\\b",
        "\\bi (think|believe|feel|am convinced|am confident|am certain|am sure) (we|that we|that this|that the|this|it)\\b",
        "\\bi (want|would like|hope|expect|anticipate) (to|us to|that we|that this)\\b",
        "\\bi (have|'ve) (decided|determined|resolved|concluded|made up my mind)\\b",
        "\\bi (am|'m) (planning|proposing|recommending|suggesting|urging|calling for)\\b",
        "\\bi (recommend|suggest|propose|urge|call for|advocate|favor|support)\\b",
        "\\bmy (plan|proposal|recommendation|suggestion|initiative|goal|objective|vision|agenda|strategy)\\b",
        "\\bthis is my (plan|proposal|initiative|goal|vision)\\b",
        "\\bunder my (leadership|direction|guidance|administration)\\b",
        "\\bi (will|shall) (do|make|take|give|provide|ensure|see to it)\\b",
        "\\bi (have|'ve) (taken|made|given|done|initiated|started|begun|launched)\\b",
        "\\bi (addressed|visited|met|spoke|talked|presented|announced|signed|called|ordered|directed|led|conducted|organized|hosted|convened|chaired|launched|initiated|established|created|built|developed|achieved|accomplished|completed|finished|delivered|sent|submitted|issued)\\b",
        "\\bi (address|visit|meet|speak|talk|present|announce|sign|call|order|direct|lead|conduct|organize|host|convene|chair|launch|initiate|establish|create|build|develop|achieve|accomplish|complete|finish|deliver|send|submit|issue)\\b",
        "\\bi call (on|upon)\\b",
        "\\bi (salute|welcome|greet|congratulate|commend|thank|acknowledge)\\b",
        "\\blet me (salute|welcome|greet|congratulate|commend|thank|acknowledge|address|explain|clarify|begin|start|say|tell|remind|emphasize|stress|point out|note|add|turn)\\b",
        "\\bi know (that|what|how|we|the)\\b",
        "\\bi see (this|that|it|an|a|the)\\b",
        "\\bi understand (that|the|this)\\b",
        "\\bi (commit|pledge|promise|vow|dedicate)\\b",
        "\\bi can (assure|tell|say)\\b",
        "\\bi ask (you|that|congress|the)\\b",
        "\\bi look forward\\b",
        "\\bi (come|stand|speak) (here|before|to)\\b",
        "\\bi join\\b",
        "\\bi share\\b",
        "\\btoday i\\b",
        "\\bi (am|'m) (here|pleased|delighted|happy) to\\b"
      )

      # Condition B: Self as authority figure
      b_patterns <- c(
        "\\bif it were up to me\\b",
        "\\bif i had my way\\b",
        "\\bif it was up to me\\b",
        "\\blet me\\b",
        "\\ballow me to\\b",
        "\\bpermit me to\\b",
        "\\bi (must|have to|need to|should) (explain|clarify|point out|emphasize|stress|note|add|say)\\b",
        "\\bmy (position|decision|opinion|view|judgment|assessment|conclusion|recommendation) (was|is|has been) (accepted|adopted|approved|endorsed|followed|implemented|recognized)\\b",
        "\\bi (have|hold|exercise|maintain|possess) (authority|power|control|responsibility|oversight)\\b",
        "\\bi (am|was) (in charge|in control|responsible|accountable|the one)\\b",
        "\\bunder my (command|control|authority|supervision|watch)\\b",
        "\\bi (direct|order|command|instruct|authorize|approve|decide|determine)\\b",
        "\\bi (made|make) the (decision|call|choice|determination)\\b",
        "\\bthe (decision|choice|call) (is|was|will be) mine\\b",
        "\\bi (take|accept|bear|assume) (responsibility|credit|blame)\\b",
        "\\bas (president|leader|commander|head|chairman|director|secretary)\\s+i\\b",
        "\\bi\\s+as\\s+(president|leader|commander|head|chairman|director|secretary)\\b"
      )

      # Condition C: Self as recipient of positive reward/recognition
      c_patterns <- c(
        "\\bi (am|'m|was|feel|felt) (honored|honoured|privileged|proud|grateful|thankful|blessed|humbled|flattered|pleased|delighted|thrilled)\\b",
        "\\bhonor(ed|s)? me\\b",
        "\\bthank(s|ful|ed|ing)? (to )?(me|myself)\\b",
        "\\byou (flatter|honor|compliment|praise) me\\b",
        "\\bpraise(d|s)? (for )?me\\b",
        "\\bcompliment(s|ed)? (me|my)\\b",
        "\\baward(ed)? (to )?(me|myself)\\b",
        "\\bprivileg(e|ed) (of|to)\\b",
        "\\bi (was|am|have been|'ve been) (chosen|selected|elected|appointed|named|designated|honored|recognized|rewarded|praised|commended|acknowledged|thanked)\\b",
        "\\bchose me\\b",
        "\\bselect(ed)? me\\b",
        "\\brecogniz(e|ed|es|ing) (me|my)\\b",
        "\\bcommend(s|ed|ing)? (me|my)\\b",
        "\\backnowledg(e|ed|es|ing) (me|my)\\b",
        "\\bit (is|was) (a|an|my) (honor|privilege|pleasure)\\b",
        "\\bi (am|'m) (deeply|truly|very|most) (honored|grateful|thankful|privileged)\\b"
      )

      # Check all patterns
      a_match <- any(sapply(a_patterns, function(p) stringr::str_detect(s, p)))
      b_match <- any(sapply(b_patterns, function(p) stringr::str_detect(s, p)))
      c_match <- any(sapply(c_patterns, function(p) stringr::str_detect(s, p)))

      if (a_match || b_match || c_match) "SC" else "OSC"
    }

    # Classify each pronoun by sentence context
    pronoun_classified <- pronoun_df |>
      dplyr::rowwise() |>
      dplyr::mutate(conf = classify_conf(sentence)) |>
      dplyr::ungroup()

    # Count SC vs OSC labels
    output <- pronoun_classified |>
      dplyr::count(conf) |>
      tidyr::complete(conf = c("SC", "OSC"), fill = list(n = 0)) |>
      tidyr::pivot_wider(names_from = conf, values_from = n) |>
      dplyr::select(SC, OSC)

    if (!bootstrap) {
      return(output)
    } else {
      count_by_sentence <- pronoun_classified |>
        dplyr::group_by(sentence_id) |>
        dplyr::count(conf) |>
        tidyr::complete(conf = c("SC", "OSC"), fill = list(n = 0)) |>
        tidyr::pivot_wider(names_from = conf, values_from = n) |>
        dplyr::select(sentence_id, SC, OSC)

      boot_results <- purrr::map_dfr(1:B, ~{
        sampled_id <- sentences$sentence_id[sample(nrow(sentences), replace = TRUE)]
        sample_df <- tibble::tibble(sentence_id = sampled_id)

        own_sentences_sampled <- sample_df |>
          dplyr::inner_join(count_by_sentence, by = "sentence_id", relationship = "many-to-many")

        if (nrow(own_sentences_sampled) == 0) {
          tibble::tibble(SC = 0, OSC = 0)
        } else {
          own_sentences_sampled |>
            dplyr::summarise(SC = sum(SC), OSC = sum(OSC))
        }
      })

      boot_results |>
        dplyr::summarise(meanSC = mean(SC), meanOSC = mean(OSC),
                        varSC = stats::var(SC), varOSC = stats::var(OSC))
    }
  }
}

# Control

# OC verb list - based on codebook
oc_verbs <- c(
  # Core codebook examples
  "believe", "consider", "feel", "think", "suspect", "wonder", "wish", "hope",
  "know", "assume", "regard", "hear", "see", "regret", "like", "dislike",
  "prefer", "mean", "want", "understand", "doubt",

  # Additional thinking/cognitive verbs
  "acknowledge", "recognize", "accept", "conclude", "contend", "estimate", "judge",
  "suppose", "expect", "imagine", "predict", "project", "assess", "anticipate",
  "contemplate", "reflect", "ponder", "realize", "comprehend", "fathom", "grasp",
  "perceive", "conceive", "deem", "reckon", "figure", "guess", "speculate",
  "theorize", "hypothesize", "surmise", "infer", "deduce", "reason",

  # Feeling/emotional verbs
  "love", "hate", "enjoy", "appreciate", "fear", "dread", "worry",
  "desire", "need", "crave", "yearn", "admire", "respect", "resent", "envy",
  "pity", "sympathize", "empathize", "mourn", "grieve", "lament", "rejoice",
  "celebrate", "cherish", "treasure", "loathe", "despise", "abhor", "detest",
  "tolerate", "endure", "suffer", "mind", "care",

  # Sensory/perception verbs
  "observe", "note", "sense", "witness", "watch", "look", "listen", "smell",
  "taste", "touch", "notice", "spot", "detect", "discern", "glimpse", "view",
  "behold", "read", "scan", "survey",

  # Communication verbs (expressing thoughts/beliefs)
  "say", "tell", "speak", "talk", "report", "mention", "remark",
  "comment", "point", "stress", "emphasize", "highlight", "add",
  "agree", "disagree", "argue", "assert", "maintain", "affirm", "deny", "claim",
  "hold", "state", "express", "declare", "announce", "profess", "confess",
  "admit", "concede", "insist", "contend", "allege", "suggest", "imply",
  "indicate", "intend", "explain", "clarify", "describe", "define",
  "repeat", "restate", "reiterate", "remind", "assure", "reassure", "promise",
  "warn", "caution", "advise", "counsel", "urge", "encourage", "discourage",
  "recommend", "propose", "submit", "request", "ask", "inquire", "question",
  "answer", "respond", "reply", "address", "discuss", "debate",
  "dispute", "challenge", "contest", "criticize", "praise", "thank",
  "congratulate", "welcome", "greet", "honor",

  # Mental state verbs
  "remember", "forget", "recall", "recollect", "identify",
  "learn", "study", "examine", "investigate", "explore", "research",

  # Stative/being verbs
  "remain", "stay", "continue", "last", "persist", "exist", "live",
  "stand", "sit", "lie", "rest", "wait", "depend", "rely", "belong",
  "consist", "comprise", "include", "contain", "involve", "require",
  "deserve", "merit", "warrant", "lack", "miss", "lose", "fail",

  # Modal-like and appearance verbs
  "seem", "appear", "tend", "prove", "turn",

  # Verbs expressing position/stance (not action)
  "favor", "oppose", "support", "endorse", "back", "side",
  "embrace", "reject", "resist", "refuse", "welcome",
  "condemn", "applaud", "commend", "salute"
)

#' Compute control (need for power) scores
#'
#' Classifies verbs associated with first-person subjects as
#' instrumental-control (IC) or other-control (OC) using dependency parsing
#' to link subjects to their governing verbs.
#'
#' @param own_entity A character vector of entity names representing the
#'   speaker's own country or group.
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{IC} and \code{OC}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanIC}, \code{meanOC}, \code{varIC}, \code{varOC}.
#' @export
get_ctrl <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  # Build entities corpus
  all_entities_corpus <- build_entities_corpus()

  ## Define own pronouns (I/we and variants)
  own_pronouns <- c("i", "we")

  ## Expanded self-reference nouns (possessive constructs)
  own_entity_terms <- unique(tolower(c(own_entity, expand_aliases_country(own_entity))))
  self_nouns <- c("government", "nation", "country", "administration",
                   "state", "party", "congress", "people")

  # Parse text
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)
  parsed <- parsed |> dplyr::mutate(token_lower = tolower(token))

  sentences <- parsed |>
    dplyr::group_by(sentence_id) |>
    dplyr::summarise(sentence = paste(token, collapse = " "), .groups = "drop")

  # Find verbs where I/we OR self-referent noun is the subject
  # Step 1a: Find all pronouns I/we that are subjects (nsubj or nsubjpass)
  pronoun_subjects <- parsed |>
    dplyr::filter(token_lower %in% own_pronouns) |>
    dplyr::filter(dep_rel %in% c("nsubj", "nsubjpass"))

  # Step 1b: Find self-referent nouns preceded by "our/my" that are subjects
  parsed <- parsed |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(prev_token = dplyr::lag(token_lower, default = "")) |>
    dplyr::ungroup()

  self_noun_subjects <- parsed |>
    dplyr::filter(
      # Possessive construct: "our government", "my country", etc.
      (token_lower %in% self_nouns & prev_token %in% c("our", "my")) |
      # Explicit country reference: "the United States", "America"
      (token_lower %in% own_entity_terms)
    ) |>
    dplyr::filter(dep_rel %in% c("nsubj", "nsubjpass"))

  # Combine both types of subjects
  all_subjects <- dplyr::bind_rows(
    pronoun_subjects |> dplyr::select(doc_id, sentence_id, token, token_id, dep_rel, head_token_id),
    self_noun_subjects |> dplyr::select(doc_id, sentence_id, token, token_id, dep_rel, head_token_id)
  )

  # Step 2: Get the verbs these subjects point to
  verbs <- parsed |> dplyr::filter(pos == "VERB")

  # Join to find verbs with self-referent subjects
  own_verbs <- all_subjects |>
    dplyr::inner_join(
      verbs |> dplyr::select(sentence_id, token_id, verb_token = token, verb_lemma = lemma, verb_tag = tag, verb_dep = dep_rel),
      by = c("sentence_id", "head_token_id" = "token_id")
    )

  if (nrow(own_verbs) == 0) {
    if (bootstrap) {
      return(tibble::tibble(meanIC = 0, meanOC = 0, varIC = 0, varOC = 0))
    } else {
      return(tibble::tibble(IC = 0, OC = 0))
    }
  }

  # Classification function
  classify_verb <- function(lemma, token, token_id, tag, dep_rel, sentence_id, parsed_df) {
    lemma <- tolower(lemma)
    token <- tolower(token)

    sentence_tokens <- parsed_df |> dplyr::filter(sentence_id == !!sentence_id)
    children <- sentence_tokens |> dplyr::filter(head_token_id == !!token_id)

    # 1. IGNORE AUXILIARIES
    if (dep_rel %in% c("aux", "auxpass")) {
      return(NA_character_)
    }

    # 2. "BE" VERBS (Comment #1) - automatically OC
    if (lemma %in% c("be", "is", "are", "was", "were", "been", "being")) {
      return("OC")
    }

    # 3. "HAVE" VERBS (Rule #3) - check for activity objects
    if (lemma %in% c("have", "has", "had")) {
      activity_nouns <- c("talks", "meeting", "meetings", "negotiation", "negotiations",
                         "dialogue", "dialogues", "engagement", "engagements",
                         "discussion", "discussions", "summit", "summits",
                         "action", "actions", "conference", "conferences",
                         "conversation", "conversations", "debate", "debates",
                         "session", "sessions", "hearing", "hearings",
                         "trip", "visit", "tour", "journey")

      has_activity_obj <- any(tolower(children$token) %in% activity_nouns &
                               children$dep_rel == "dobj")

      if (has_activity_obj) return("IC") else return("OC")
    }

    # 4. NEGATION (Rule #2) - negated action verb = IC (decision not to act)
    is_negated <- any(children$dep_rel == "neg")
    if (is_negated) {
      # Negated OC verb stays OC, negated IC verb becomes IC
      if (lemma %in% oc_verbs) return("OC") else return("IC")
    }

    # 5. THINKING / FEELING / SENSORY VERBS (Comment #1) = OC
    if (lemma %in% oc_verbs) {
      return("OC")
    }

    # 6. FINAL: All other verbs = IC (action verbs)
    return("IC")
  }

  # Apply classification to each verb with I/we subject
  verb_classified <- own_verbs |>
    dplyr::rowwise() |>
    dplyr::mutate(
      ctrl = classify_verb(verb_lemma, verb_token, head_token_id,
                          verb_tag, verb_dep, sentence_id, parsed)
    ) |>
    dplyr::ungroup() |>
    dplyr::filter(!is.na(ctrl))

  # Count and reshape
  output <- verb_classified |>
    dplyr::count(ctrl) |>
    tidyr::complete(ctrl = c("IC", "OC"), fill = list(n = 0)) |>
    tidyr::pivot_wider(names_from = ctrl, values_from = n)

  if (bootstrap) {
    if (nrow(verb_classified) == 0) {
      btres <- tibble::tibble(meanIC = 0, meanOC = 0, varIC = 0, varOC = 0)
    } else {
      sentence_summary <- verb_classified |>
        dplyr::group_by(sentence_id) |>
        dplyr::count(ctrl) |>
        tidyr::complete(ctrl = c("IC", "OC"), fill = list(n = 0)) |>
        tidyr::pivot_wider(names_from = ctrl, values_from = n)

      boot_results <- purrr::map_dfr(1:B, ~{
        sampled_id <- sentences$sentence_id[sample(nrow(sentences), replace = TRUE)]
        sample_df <- tibble::tibble(sentence_id = sampled_id)

        own_sentences_sampled <- sample_df |>
          dplyr::inner_join(sentence_summary, by = "sentence_id", relationship = "many-to-many")

        if (nrow(own_sentences_sampled) == 0) {
          tibble::tibble(IC = 0, OC = 0)
        } else {
          own_sentences_sampled |>
            dplyr::summarise(IC = sum(IC), OC = sum(OC))
        }
      })

      btres <- boot_results |>
        dplyr::summarize(meanIC = mean(IC), meanOC = mean(OC)) |>
        dplyr::mutate(varIC = stats::var(boot_results$IC), varOC = stats::var(boot_results$OC))
    }

    return(btres)
  } else {
    return(output)
  }
}

# Nationalism

# Modifier lists for own-country references
favorable_descriptors <- c(
  "great", "good", "peace-loving", "progressive", "successful", "prosperous",
  "democratic", "leading", "innovative", "vibrant", "dynamic", "respected",
  "excellent", "positive", "constructive", "friendly", "free",
  "proud", "diverse", "thriving", "strong", "noble", "beautiful", "resilient",
  "hardworking", "brave", "courageous", "united", "powerful", "mighty",
  "blessed", "fortunate", "exceptional", "magnificent", "glorious")

strength_descriptors <- c(
  "powerful", "capable", "prepared", "strong", "resilient", "resourceful",
  "unyielding", "robust", "effective", "decisive", "leading", "advanced",
  "determined", "steadfast", "unwavering", "formidable", "invincible")

# Phrases indicating national honor/identity (checked in sentence context)
national_honor_phrases <- c(
  "sovereignty", "self-determination", "national interest", "homeland",
  "patriotism", "patriotic", "national pride", "founding principles",
  "heritage", "tradition", "honor", "duty", "sacred", "independence",
  "freedom", "liberty", "destiny", "unity",
  # Additional nationalistic phrases
  "american dream", "american people", "american spirit", "american way",
  "our great nation", "this great nation", "our beloved country",
  "land of the free", "home of the brave", "greatest nation",
  "our founding fathers", "founding principles", "constitution",
  "stars and stripes", "old glory", "star-spangled",
  "god bless america", "god bless the united states",
  "defend our nation", "protect our nation", "serve our nation",
  "proud american", "proud to be american", "american values",
  "national security", "national defense", "national sovereignty",
  "american leadership", "american strength", "american resolve",
  "our troops", "our soldiers", "our armed forces", "our military",
  "brave men and women", "heroes", "veterans",
  # Generic nation-referent phrases (Fix 2: expanded list)
  "our nation", "my nation", "this nation", "the nation",
  "our country", "my country", "this country",
  "our government", "our people", "our citizens",
  "great nation", "proud nation", "best nation",
  "our strength", "our power", "our future",
  "our way of life", "our values", "our children",
  "stand united", "stand together",
  "our allies", "our friends", "our partners",
  "defend our", "protect our", "serve our",
  "fight for", "work for our",
  "democratic party", "republican party", "our party",
  "our state", "our government's"
)

# Unfavorable descriptors for other countries
unfavorable_descriptors_others <- c(
  "weak", "colonialist", "imperialistic", "hostile", "unfriendly", "corrupt", "failing",
  "aggressive", "ineffective", "powerless", "backward", "oppressive", "dangerous",
  "destructive", "imperialist", "authoritarian", "tyrannical", "repressive", "fascist",
  "totalitarian", "genocidal", "terrorist", "pariah", "illegitimate", "extremist",
  "undemocratic", "autocratic", "theocratic", "dictatorial", "belligerent", "evil",
  "rogue", "brutal", "ruthless", "criminal", "barbaric")

# Meddling behavior verbs for other countries
meddling_verbs <- c(
  "interfere", "meddle", "undermine", "destabilize", "violate", "threaten",
  "attack", "invade", "occupy", "exploit", "manipulate", "deceive", "provoke",
  "sabotage", "subvert", "infiltrate", "coerce", "intimidate", "oppress")

# Own-entity pronouns
own_pronouns <- c("we", "us", "our", "ours", "ourselves")
other_pronouns <- c("they", "them", "their", "theirs", "themselves")

# International organizations to count as "Other" entities for nationalism
intl_orgs <- c(
  "united nations", "un", "nato", "who", "world health organization",
  "imf", "international monetary fund", "world bank", "oas",
  "organization of american states", "european union", "eu", "ec",
  "european community", "g7", "g8", "g20", "opec", "wto",
  "world trade organization", "iaea", "international atomic energy agency",
  "security council", "general assembly"
)

#' Compute nationalism scores
#'
#' Classifies entity references in the text as nationalistic (N) or
#' other-nationalistic (ON) based on own/other entity detection and
#' contextual modifiers at the referent level.
#'
#' @param own_entity A character vector of entity names representing the
#'   speaker's own country or group.
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{N} and \code{ON}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanN}, \code{meanON}, \code{varN}, \code{varON}.
#' @export
get_nat <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  # Define own entity terms
  own_entity_terms <- unique(tolower(c(own_entity, expand_aliases_country(own_entity))))

  # Parse text
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)
  parsed <- parsed |> dplyr::mutate(token_lower = tolower(token))

  # Get sentence text for context checking
  sentences <- parsed |>
    dplyr::group_by(sentence_id) |>
    dplyr::summarise(sentence = paste(token, collapse = " "), .groups = "drop")

  # STEP 1: Group multi-word entities into single referents
  # Use run-length encoding to group consecutive entity tokens
  # Only include GPE (countries) - ORG caused massive overcounting
  parsed <- parsed |>
    dplyr::mutate(
      is_gpe = grepl("GPE", entity),
      ent_group = data.table::rleid(is_gpe & !is.na(entity) & entity != "")
    )

  # Extract GPE entity phrases (grouped)
  gpe_entities <- parsed |>
    dplyr::filter(is_gpe) |>
    dplyr::group_by(doc_id, sentence_id, ent_group) |>
    dplyr::summarise(
      phrase = paste(token_lower, collapse = " "),
      first_token_id = dplyr::first(token_id),
      .groups = "drop"
    ) |>
    dplyr::left_join(sentences, by = "sentence_id")

  # STEP 2: Count own-reference pronouns - ONE per sentence (not per pronoun)
  # Pronouns only contribute to N (with nationalistic context), never to ON
  own_pronoun_refs <- parsed |>
    dplyr::filter(
      token_lower %in% c("we", "our") &
      dep_rel %in% c("nsubj", "poss", "nsubjpass")
    ) |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::summarise(
      phrase = "we/our",
      first_token_id = dplyr::first(token_id),
      .groups = "drop"
    ) |>
    dplyr::left_join(sentences, by = "sentence_id")

  # STEP 3: Classify referents

  # Classify GPE entities as own or other
  if (nrow(gpe_entities) > 0) {
    gpe_entities <- gpe_entities |>
      dplyr::mutate(
        entity_type = ifelse(phrase %in% own_entity_terms, "own", "other"),
        is_pronoun = FALSE
      )
  } else {
    gpe_entities <- tibble::tibble(
      sentence_id = integer(), phrase = character(), first_token_id = integer(),
      sentence = character(), entity_type = character(), is_pronoun = logical()
    )
  }

  # Own pronouns are always "own" type and marked as pronouns
  if (nrow(own_pronoun_refs) > 0) {
    own_pronoun_refs <- own_pronoun_refs |>
      dplyr::mutate(entity_type = "own", is_pronoun = TRUE)
  } else {
    own_pronoun_refs <- tibble::tibble(
      sentence_id = integer(), phrase = character(), first_token_id = integer(),
      sentence = character(), entity_type = character(), is_pronoun = logical()
    )
  }

  # Combine all referents
  all_referents <- dplyr::bind_rows(
    gpe_entities |> dplyr::select(sentence_id, phrase, first_token_id, sentence, entity_type, is_pronoun),
    own_pronoun_refs |> dplyr::select(sentence_id, phrase, first_token_id, sentence, entity_type, is_pronoun)
  )

  if (nrow(all_referents) == 0) {
    if (bootstrap) {
      return(tibble::tibble(meanN = 0, meanON = 0, varN = 0, varON = 0))
    } else {
      return(tibble::tibble(N = 0, ON = 0))
    }
  }

  # STEP 4: Apply N/ON classification based on context
  classify_referent <- function(token_id, sent_id, ent_type, phrase, sentence_text, parsed_df, is_pron) {

    sent_tokens <- parsed_df |>
      dplyr::filter(sentence_id == sent_id) |>
      dplyr::mutate(token_lower = tolower(token))

    # Find modifiers attached to this token via dependency
    modifiers <- sent_tokens |>
      dplyr::filter(head_token_id == token_id & dep_rel %in% c("amod", "advmod", "compound"))

    modifier_words <- tolower(modifiers$token)

    # Check window around token
    token_idx <- which(sent_tokens$token_id == token_id)
    if (length(token_idx) > 0) {
      window_start <- max(1, token_idx - 3)
      window_end <- min(nrow(sent_tokens), token_idx + 3)
      window_tokens <- tolower(sent_tokens$token[window_start:window_end])
    } else {
      window_tokens <- character(0)
    }

    context_words <- unique(c(modifier_words, window_tokens))
    sentence_lower <- tolower(sentence_text)

    if (ent_type == "own") {
      # Check for favorable/strength modifiers or national honor phrases
      has_favorable <- any(context_words %in% favorable_descriptors)
      has_strength <- any(context_words %in% strength_descriptors)
      has_honor <- any(sapply(national_honor_phrases, function(p) grepl(p, sentence_lower, fixed = TRUE)))

      if (has_favorable || has_strength || has_honor) {
        return("N")
      } else {
        return("ON")
      }
    } else {
      # entity_type == "other"
      has_unfavorable <- any(context_words %in% unfavorable_descriptors_others)
      sent_verbs <- tolower(sent_tokens$lemma[sent_tokens$pos == "VERB"])
      has_meddling <- any(sent_verbs %in% meddling_verbs)

      if (has_unfavorable || has_meddling) {
        return("N")
      } else {
        return("ON")
      }
    }
  }

  # Apply classification
  referent_classified <- all_referents |>
    dplyr::rowwise() |>
    dplyr::mutate(
      mod = classify_referent(first_token_id, sentence_id, entity_type, phrase, sentence, parsed, is_pronoun)
    ) |>
    dplyr::ungroup()

  # STEP 5: Count N and ON
  output <- referent_classified |>
    dplyr::count(mod) |>
    tidyr::complete(mod = c("N", "ON"), fill = list(n = 0)) |>
    tidyr::pivot_wider(names_from = mod, values_from = n)

  if (bootstrap) {
    sentence_summary <- referent_classified |>
      dplyr::group_by(sentence_id) |>
      dplyr::count(mod) |>
      tidyr::complete(mod = c("N", "ON"), fill = list(n = 0)) |>
      tidyr::pivot_wider(names_from = mod, values_from = n)

    boot_results <- purrr::map_dfr(1:B, ~{
      sampled_ids <- sample(unique(sentences$sentence_id), replace = TRUE)
      sample_df <- tibble::tibble(sentence_id = sampled_ids)

      sampled <- sample_df |>
        dplyr::inner_join(sentence_summary, by = "sentence_id", relationship = "many-to-many")

      if (nrow(sampled) == 0) {
        tibble::tibble(N = 0, ON = 0)
      } else {
        tibble::tibble(N = sum(sampled$N, na.rm = TRUE), ON = sum(sampled$ON, na.rm = TRUE))
      }
    })

    btres <- boot_results |>
      dplyr::summarise(meanN = mean(N), meanON = mean(ON)) |>
      dplyr::mutate(varN = stats::var(boot_results$N), varON = stats::var(boot_results$ON))

    return(btres)
  } else {
    return(output)
  }
}

# Distrust

# Distrust verbs - focused on actual distrust/harmful intent
distrust_verbs <- c(
  # Core harmful action verbs (high confidence)
  "force", "harm", "hurt", "damage", "undermine", "threaten",
  "interfere", "plot", "scheme", "sabotage", "subvert",
  "deceive", "manipulate", "destroy", "attack", "conspire",
  "exploit", "occupy", "infiltrate", "coerce", "intimidate",
  "blackmail", "extort", "oppress", "bully", "terrorize",
  "invade", "violate", "endanger", "jeopardize",
  # Distrust/suspicion verbs (high confidence)
  "distrust", "suspect", "mistrust", "fear",
  # Strong condemnation verbs
  "condemn", "denounce", "accuse", "blame"
)

# Harmful nominals
harmful_nominals <- c(
  # Clear harmful outcomes
  "trouble", "harm", "damage", "instability", "chaos", "violence", "unrest",
  "conflict", "conflicts", "war", "warfare",
  "famine", "suffering", "misery", "terror", "attack", "attacks", "aggression",
  "threat", "threats", "danger", "dangers", "sabotage", "subversion",
  # Hostile actions/states
  "interference", "propaganda", "disinformation", "espionage", "infiltration",
  "oppression", "occupation", "destabilization",
  "hostility", "animosity", "enmity", "hatred",
  # Clear distrust indicators
  "suspicion", "distrust", "mistrust"
)

# Build regex patterns
harmful_re <- paste0("(?:", paste(harmful_nominals, collapse = "|"), ")")
harmful_verbs_re <- paste0("(?:", paste(distrust_verbs, collapse = "|"), ")")

pat_light_verb <- paste0(
  "\\b(cause|create|bring|inflict|incite|spark|fuel|foment|provoke|stir|trigger)\\b",
  "\\s+(?:\\w+\\s+){0,2}", harmful_re, "\\b"
)
pat_pose_threat <- "\\b(pose|present|constitute)\\b\\s+(?:an?\\s+)?(threat|risk|danger)s?\\b"
pat_engage_in <- paste0(
  "\\b(engage\\s+in|carry\\s+out|conduct|mount|wage|launch)\\b\\s+",
  "(?:attack|attacks|sabotage|espionage|aggression|war|warfare)\\b"
)
pat_attempt_to <- paste0(
  "\\b(attempt|seek|plan|prepare|conspire)\\b\\s+to\\s+", harmful_verbs_re, "\\b"
)
pat_any_harm <- paste(pat_light_verb, pat_pose_threat, pat_engage_in, pat_attempt_to, sep = "|")

# CODEBOOK-ALIGNED target nouns list
target_nouns <- c(
  # CODEBOOK SPECIFIED nouns (from distrust.rtf)
  "country", "countries", "people", "government", "governments",
  "state", "states", "nation", "nations", "society", "societies",
  "power", "powers", "political party", "party", "parties",
  "rebel", "rebels", "revolutionary", "revolutionaries",
  # Additional entity types from codebook
  "opposition", "militants", "faction", "factions", "agents", "spies", "mercenaries",
  # Inherently hostile entities (always relevant for distrust coding)
  "enemy", "enemies", "adversary", "adversaries", "foe", "foes",
  "terrorist", "terrorists", "attacker", "attackers", "aggressor", "aggressors",
  "dictator", "dictators", "tyrant", "tyrants", "regime", "regimes",
  # Military entities
  "army", "military", "forces", "troops",
  # Generic group terms (from codebook)
  "group", "groups", "organization", "organizations", "movement", "movements"
)

own_pronouns <- c("i", "me", "my", "mine", "myself", "we", "us", "our", "ours", "ourselves")

#' Compute distrust scores
#'
#' Performs per-entity classification of references in the text as
#' suspicious (S) or other-suspicious (OS) based on distrust verbs,
#' harmful nominals, and contextual modifiers.
#'
#' @param own_entity A character vector of entity names representing the
#'   speaker's own country or group.
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{S} and \code{OS}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanS}, \code{varS}, \code{meanOS}, \code{varOS}.
#' @export
get_dist <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  # Build country list
  country_list <- build_country_list()

  # Own/other corpus
  io_vec <- if (exists("io", inherits = TRUE)) get("io") else character(0)

  own_entity <- tolower(own_entity)
  own_entity <- unique(c(own_entity, expand_aliases_country(own_entity)))

  # US states - these are domestic entities for a US speaker
  us_states <- tolower(c(
    "alabama", "alaska", "arizona", "arkansas", "california", "colorado", "connecticut",
    "delaware", "florida", "georgia", "hawaii", "idaho", "illinois", "indiana", "iowa",
    "kansas", "kentucky", "louisiana", "maine", "maryland", "massachusetts", "michigan",
    "minnesota", "mississippi", "missouri", "montana", "nebraska", "nevada", "new hampshire",
    "new jersey", "new mexico", "new york", "north carolina", "north dakota", "ohio",
    "oklahoma", "oregon", "pennsylvania", "rhode island", "south carolina", "south dakota",
    "tennessee", "texas", "utah", "vermont", "virginia", "washington", "west virginia",
    "wisconsin", "wyoming", "district of columbia", "d.c.", "dc", "puerto rico", "guam"
  ))

  # US government bodies - domestic entities
  us_gov_orgs <- tolower(c(
    "congress", "senate", "house", "house of representatives", "supreme court",
    "white house", "pentagon", "state department", "treasury", "defense department",
    "justice department", "fbi", "cia", "nsa", "homeland security",
    "democratic party", "republican party", "democrats", "republicans",
    "administration", "cabinet", "federal reserve", "fed"
  ))

  # US demonyms and references
  us_demonyms <- tolower(c(
    "american", "americans", "u.s.", "u.s.a.", "usa"
  ))

  own_terms <- unique(tolower(c(
    own_entity, us_states, us_gov_orgs, us_demonyms,
    "we", "our nation", "our country", "us", "our state", "our government", "our institution",
    "our group", "our party", "our people", "i", "my nation", "my country", "me", "my state",
    "my government", "my institution", "my group", "my party", "my people"
  )))

  all_entities_corpus <- unique(tolower(c(io_vec, country_list)))

  other_terms <- unique(c(setdiff(all_entities_corpus, own_terms),
                          "they", "their nation", "their country", "their state",
                          "their government", "their institution", "their party", "their people"))

  # Parse
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)
  parsed <- parsed |> dplyr::mutate(
    token_lower = tolower(token),
    lemma_lower = tolower(lemma)
  )

  # Track quote spans (but don't skip sentences)
  parsed <- parsed |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(
      is_quote_char = token %in% c('"', "'", "\u201C", "\u201D", "\u2018", "\u2019"),
      quote_cumsum = cumsum(is_quote_char),
      in_quote = (quote_cumsum %% 2 == 1)
    ) |>
    dplyr::ungroup()

  # Entity grouping (multiword)
  parsed <- parsed |>
    dplyr::mutate(ent_flag = ifelse(!is.na(entity) & entity != "", 1, NA)) |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(entity_group_id = data.table::rleid(ent_flag)) |>
    dplyr::ungroup() |>
    dplyr::mutate(entity_group_id = ifelse(is.na(ent_flag), NA, entity_group_id))

  multiword_entities <- parsed |>
    dplyr::filter(!is.na(entity_group_id)) |>
    dplyr::group_by(doc_id, sentence_id, entity_group_id) |>
    dplyr::reframe(
      ent_phrase = paste(token_lower, collapse = " "),
      ent_type = dplyr::first(na.omit(entity)),
      .groups = "drop"
    )

  parsed <- parsed |>
    dplyr::left_join(multiword_entities, by = c("doc_id", "sentence_id", "entity_group_id"))

  # Get sentence text
  sentences <- parsed |>
    dplyr::group_by(sentence_id) |>
    dplyr::summarise(sentence_text = paste(token, collapse = " "), .groups = "drop")

  # PER-ENTITY classification
  classify_entity <- function(entity_row, sentence_tokens, sentence_text_df) {
    token_id <- entity_row$token_id
    sent_id <- entity_row$sentence_id

    # Get full sentence text
    sent_text <- sentence_text_df |>
      dplyr::filter(sentence_id == sent_id) |>
      dplyr::pull(sentence_text)
    sent_text_lower <- tolower(sent_text)

    # Get all tokens in sentence
    sent_tokens <- sentence_tokens |>
      dplyr::filter(sentence_id == sent_id)

    # Get all lemmas in sentence
    sent_lemmas <- sent_tokens$lemma_lower

    # Check 1: Direct distrust verbs anywhere in sentence
    has_distrust_verb <- any(sent_lemmas %in% distrust_verbs)

    # Check 2: Harmful nominals anywhere in sentence
    has_harmful_nominal <- any(sent_lemmas %in% harmful_nominals)

    # Check 3: Light verb + harm patterns in sentence text
    has_harm_pattern <- stringr::str_detect(sent_text_lower, stringr::regex(pat_any_harm, ignore_case = TRUE))

    # Check 4: Entity-specific negative modifiers
    entity_modifiers <- sent_tokens |>
      dplyr::filter(head_token_id == token_id & dep_rel %in% c("amod", "compound"))

    negative_modifiers <- c("hostile", "aggressive", "dangerous", "evil", "corrupt",
                           "threatening", "menacing", "suspicious", "dubious", "rogue",
                           "brutal", "terrorist", "criminal", "oppressive", "belligerent")
    has_negative_modifier <- any(tolower(entity_modifiers$token) %in% negative_modifiers)

    # Check 5: Sentence contains inherently negative entity types
    entity_phrase <- tolower(entity_row$noun_phrase)
    inherently_distrusted <- c("enemy", "enemies", "terrorist", "terrorists", "attacker",
                               "attackers", "aggressor", "aggressors", "adversary", "adversaries",
                               "foe", "foes", "dictator", "dictators", "tyrant", "tyrants")
    is_inherently_distrusted <- entity_phrase %in% inherently_distrusted

    # Determine S or OS
    if (has_distrust_verb || has_harmful_nominal || has_harm_pattern ||
        has_negative_modifier || is_inherently_distrusted) {
      return("S")
    } else {
      return("OS")
    }
  }

  # Find all codeable entities
  parsed <- parsed |>
    dplyr::mutate(entity_clean = stringr::str_replace(entity, "_[BI]$", ""))

  # Identify self-referential target nouns by checking preceding tokens
  self_ref_preceding <- c("our", "my", "this", "these")

  parsed <- parsed |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(
      prev_token = dplyr::lag(token_lower, default = ""),
      is_self_ref_noun = (lemma_lower %in% target_nouns) & (prev_token %in% self_ref_preceding)
    ) |>
    dplyr::ungroup()

  # BALANCED entity detection
  parsed <- parsed |>
    dplyr::mutate(noun_phrase_tmp = dplyr::coalesce(ent_phrase, token_lower))

  # US demonyms/domestic NORP to exclude
  us_norp <- tolower(c("american", "americans", "democratic", "republican",
                        "texan", "texans", "californian", "californians",
                        "new yorker", "new yorkers", "floridian", "floridians"))

  entities <- parsed |>
    dplyr::filter(
      # GPE entities - ONLY if they match country_list (foreign countries)
      (entity_clean == "GPE" & noun_phrase_tmp %in% all_entities_corpus) |
      # ORG entities - ONLY if they match io list (international organizations)
      (entity_clean == "ORG" & noun_phrase_tmp %in% io_vec) |
      # NORP entities - but exclude US demonyms
      (entity_clean == "NORP" & !(noun_phrase_tmp %in% us_norp) & !(noun_phrase_tmp %in% own_terms)) |
      # Target nouns from codebook - ONLY if NOT self-referential
      (lemma_lower %in% target_nouns & !is_self_ref_noun)
    ) |>
    # Use multi-word phrase if available
    dplyr::mutate(noun_phrase = noun_phrase_tmp) |>
    # Filter out speaker's own terms
    dplyr::filter(!(noun_phrase %in% own_terms))

  if (nrow(entities) == 0) {
    if (!bootstrap) return(tibble::tibble(S = 0L, OS = 0L))
    return(tibble::tibble(meanS = 0, varS = 0, meanOS = 0, varOS = 0))
  }

  # Apply per-entity classification
  entity_classified <- entities |>
    dplyr::rowwise() |>
    dplyr::mutate(label = classify_entity(dplyr::pick(dplyr::everything()), parsed, sentences)) |>
    dplyr::ungroup() |>
    dplyr::select(sentence_id, noun = noun_phrase, label)

  # Count (NO deduplication - count each mention)
  if (!bootstrap) {
    output <- entity_classified |>
      dplyr::count(label) |>
      tidyr::complete(label = c("S", "OS"), fill = list(n = 0)) |>
      tidyr::pivot_wider(names_from = label, values_from = n) |>
      dplyr::select(S, OS)

    return(output)

  } else {
    count_by_sentence <- entity_classified |>
      dplyr::group_by(sentence_id) |>
      dplyr::count(label) |>
      tidyr::complete(label = c("S", "OS"), fill = list(n = 0)) |>
      tidyr::pivot_wider(names_from = label, values_from = n) |>
      dplyr::select(sentence_id, S, OS)

    boot_results <- purrr::map_dfr(1:B, ~{
      sampled_ids <- sample(unique(sentences$sentence_id), replace = TRUE)
      sample_df <- tibble::tibble(sentence_id = sampled_ids)

      sampled <- sample_df |>
        dplyr::inner_join(count_by_sentence, by = "sentence_id", relationship = "many-to-many")

      if (nrow(sampled) == 0) {
        tibble::tibble(S = 0, OS = 0)
      } else {
        tibble::tibble(S = sum(sampled$S, na.rm = TRUE), OS = sum(sampled$OS, na.rm = TRUE))
      }
    })

    btres <- tibble::tibble(
      meanS = mean(boot_results$S, na.rm = TRUE),
      varS = stats::var(boot_results$S, na.rm = TRUE),
      meanOS = mean(boot_results$OS, na.rm = TRUE),
      varOS = stats::var(boot_results$OS, na.rm = TRUE)
    )

    return(btres)
  }
}

# Conceptual complexity

# HC STEMS - Full original list
hc_stems <- c(
  # Core HC words from codebook
  "admit", "almost", "also", "ambigu", "approximat", "aspect",
  "chang", "circumstanc", "clarif", "condition", "consid", "depend", "differ",
  "distinguish", "either", "except", "further", "general", "gradual", "however",
  "impl", "indicat", "intend", "mainly", "may", "maybe", "might",
  "moreover", "often", "open",
  "partial", "particul", "perhaps", "persuad", "possib", "probab", "qualif",
  "question", "reconsid", "reexamin", "reflect", "revis", "seem",
  "tend", "trend", "uncertain", "vari",

  # Additional HC stems for better coverage
  "alternativ", "flexib", "nuanc", "complicat", "complex",
  "balanc", "moderat", "relativ", "tentativ", "hypothetic",
  "theoretic", "speculat", "contingent", "conceivab",
  "presum", "apparent", "plausib", "feasib", "likel",
  "integrat", "synthes", "coordinat", "multifacet", "multidimension",
  "divers", "plural", "variab", "occasion", "temporar", "intermittent",

  # Additional stems
  "usual", "typical", "fair", "quite", "rather",
  "nonetheless", "nevertheless", "overall", "potential",
  "suggest", "assum", "believ", "estimat", "evaluat",
  "arguab", "debat", "unclear", "undecid",
  "essenti", "broad", "rough", "approx", "near",
  "slight", "somewh", "marginal", "modest",
  "graduat", "progress", "evolv", "emerg", "develop",
  "adjust", "adapt", "modif", "refin", "calibrat",
  "acknowledg", "recogniz", "realiz", "understand",

  # Restore remaining stems (trying to get to near-zero bias)
  "interpret", "analyz", "assess", "examin", "review"
)

# HC phrases
hc_phrases <- c(
  # Phrases from codebook
  "among other things", "inter alia", "in the sense that", "in this case",
  "an example of", "an illustration of", "for example", "for instance",
  "to illustrate", "namely", "such as",
  "at the moment", "at times", "for a time", "from time to time", "now and then",
  "once in a while", "part of the time", "sometimes", "occasionally", "periodically",
  "e.g.", "i.e.",

  # Additional phrases from codebook
  "as far as", "as well as", "more or less", "one of the", "on the whole",
  "with respect to", "in part", "to some extent", "in some cases",
  "under certain circumstances", "depending on", "it depends",
  "in a sense", "to a degree", "to some degree", "up to a point",

  # Additional phrases
  "on the other hand", "in contrast", "by contrast", "at the same time",
  "sort of", "kind of", "at least", "in general", "as a rule",
  "for the most part", "in many ways", "in some ways", "to a certain extent",
  "in certain respects", "in other words", "that is to say",
  "it seems", "it appears", "it would seem", "it would appear",
  "may or may not", "whether or not", "one way or another",
  "on balance", "taking into account", "when considering",
  "to the extent", "insofar as", "in this context", "given the situation",
  "to a certain degree", "to a large extent", "to a great extent",
  "in large part", "in some measure",
  "subject to", "contingent on", "depending upon", "provided that",
  "assuming that", "given that", "on the assumption",
  "one possibility", "another option", "an alternative", "a different approach"

)

# HC single words that should match exactly
hc_words_exact <- c(
  "if", "whether", "some", "someone", "something", "somewhere", "somehow",
  "somewhat", "sometimes", "but", "yet", "although", "though", "unless",
  "whereas", "while", "provided", "supposing",

  # Modal verbs (restore - they indicate conditionality)
  "could", "would", "should"
)

# Low complexity list - expanded
low_complexity <- unique(c(
  # Core LC words from codebook
  "above all", "irreversible", "actually", "maximum", "minimum",
  "absolutely", "absolute", "merely", "all", "most", "always", "must",
  "any", "anyone", "anything", "anybody", "anywhere", "anyhow", "anyway",
  "necessary", "necessarily", "necessity", "avoid", "never", "best",
  "no", "no one", "nothing", "nobody", "nowhere", "none", "not even",
  "obvious", "obviously", "of course", "only", "ought",
  "complete", "completely", "comprehensive", "compulsory",
  "simply", "singly", "continual", "continually",
  "convinced", "convincing", "definite", "definitely", "determined",
  "doubtless", "each", "entirely", "entire",
  "every", "everyone", "everything", "everybody", "everywhere",
  "false", "forever", "fully", "full", "impossible",
  "unconditional", "unconditionally", "indeed", "whatever",
  "indispensable", "indisputable", "inevitable", "inevitably",
  "insist", "unalterably", "undeniable", "undeniably",
  "undoubtedly", "undoubted", "unequalled", "unqualified",
  "unquestionable", "unquestionably", "whole", "as a whole", "wholly",
  "without a doubt", "without doubt", "without a question", "without question",

  # Additional LC words
  "certain", "certainly", "certainty", "clear", "clearly",
  "pure", "purely", "solely", "sole", "total", "totally",
  "true", "truly", "truth", "compelled", "consequently",
  "regardless", "no matter what", "bound to", "cannot", "can't",
  "sure", "surely", "for sure"
))

# REGEX BUILDERS -----

#' Prepare word-stem regex pattern
#' @param stems Character vector of word stems.
#' @return A single regex string.
#' @keywords internal
prep_wordstem_regex <- function(stems) {
  stems <- stems[order(-nchar(stems))]
  stems <- stringr::str_replace_all(stems, "\\.", "\\\\.")
  stems <- stringr::str_replace_all(stems, "\\s+", "")
  paste0("(?i)\\b(", paste(stems, collapse = "|"), ")\\p{L}*\\b")
}

#' Prepare phrase regex pattern
#' @param phrases Character vector of phrases.
#' @return A single regex string.
#' @keywords internal
prep_phrase_regex <- function(phrases) {
  phrases <- phrases[order(-nchar(phrases))]
  phrases <- stringr::str_replace_all(phrases, "\\.", "\\\\.")
  phrases <- stringr::str_replace_all(phrases, "\\s+", " ")
  phrases <- stringr::str_replace_all(phrases, " ", "\\\\s+")

  paste0("(?i)\\b(", paste(phrases, collapse = "|"), ")\\b")
}

#' Prepare exact-word regex pattern
#' @param words Character vector of words to match exactly.
#' @return A single regex string.
#' @keywords internal
prep_exact_regex <- function(words) {
  words <- words[order(-nchar(words))]
  words <- stringr::str_replace_all(words, "\\.", "\\\\.")
  paste0("(?i)\\b(", paste(words, collapse = "|"), ")\\b")
}

#' Prepare low-complexity regex pattern
#' @param items Character vector of low-complexity words and phrases.
#' @return A single regex string.
#' @keywords internal
prep_lc_regex <- function(items) {
  items <- items[order(-nchar(items))]
  items <- stringr::str_replace_all(items, "\\.", "\\\\.")
  items <- stringr::str_replace_all(items, "\\s+", " ")
  items <- stringr::str_replace_all(items, " ", "\\\\s+")
  paste0("(?i)\\b(", paste(items, collapse = "|"), ")\\b")
}

hc_stem_pattern <- prep_wordstem_regex(hc_stems)
hc_phrase_pattern <- prep_phrase_regex(hc_phrases)
hc_exact_pattern <- prep_exact_regex(hc_words_exact)
lc_pattern <- prep_lc_regex(low_complexity)

# QUOTE STRIPPING -----

#' Strip quoted text from a string
#' @param x A character string.
#' @return The string with quoted passages removed.
#' @keywords internal
strip_quoted <- function(x) {
  # Remove curly quotes
  x <- stringr::str_replace_all(x, "[\u201C\u201D][^\u201C\u201D]*[\u201C\u201D]", " ")
  # Remove straight double quotes
  x <- stringr::str_replace_all(x, '"[^"]*"', " ")
  # Remove single quotes (but be careful with apostrophes)
  x <- stringr::str_replace_all(x, "'[^']*'", " ")
  x
}

# NEGATION-AWARE COUNTING -----

#' Count valid (non-negated) pattern matches
#' @param txt A character string to search.
#' @param pattern A regex pattern to match.
#' @param window_chars Integer; number of characters to look back for negation.
#' @return Integer count of valid matches.
#' @keywords internal
count_valid <- function(txt, pattern, window_chars = 40) {
  locs <- stringr::str_locate_all(txt, stringr::regex(pattern))[[1]]
  if (nrow(locs) == 0) return(0L)

  valid <- 0L
  for (i in seq_len(nrow(locs))) {
    start <- locs[i, "start"]
    left <- substr(txt, max(1, start - window_chars), start - 1)
    left <- stringr::str_to_lower(left)

    # Check for negation
    neg_ok <- stringr::str_detect(left, "\\b(not|no|never|none|n't)\\b\\s*$") ||
      stringr::str_detect(left, "\\b(not|no|never|none|n't)\\b\\s+\\b\\w+\\b\\s*$")

    if (!neg_ok) valid <- valid + 1L
  }
  valid
}

# MAIN FUNCTION -----

#' Compute conceptual complexity scores
#'
#' Counts high-complexity (HC) and low-complexity (LC) markers in the text,
#' using word stems, phrases, and exact words with negation-aware counting.
#'
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @param quote_strip Logical; if \code{TRUE}, remove quoted text before
#'   counting. Default is \code{TRUE}.
#' @param window_chars Integer; number of characters to look back for
#'   negation context. Default is 40.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{HC} and \code{LC}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanHC}, \code{varHC}, \code{meanLC}, \code{varLC}.
#' @export
get_complex <- function(text, bootstrap = FALSE, B = 1000,
                       quote_strip = TRUE, window_chars = 40) {

  # Parse & make sentence text
  parsed <- spacyr::spacy_parse(text, dependency = FALSE, lemma = FALSE)
  sentence_df <- parsed |>
    dplyr::group_by(sentence_id) |>
    dplyr::summarise(sentence_text = paste(token, collapse = " "), .groups = "drop")

  classify_complexity <- function(txt) {
    speaker_txt <- if (quote_strip) strip_quoted(txt) else txt
    speaker_txt <- stringr::str_to_lower(speaker_txt)

    # Count HC from all three sources
    HC <- count_valid(speaker_txt, hc_stem_pattern, window_chars = window_chars) +
      count_valid(speaker_txt, hc_phrase_pattern, window_chars = window_chars) +
      count_valid(speaker_txt, hc_exact_pattern, window_chars = window_chars)

    LC <- count_valid(speaker_txt, lc_pattern, window_chars = window_chars)

    list(HC = HC, LC = LC)
  }

  results <- sentence_df |>
    dplyr::rowwise() |>
    dplyr::mutate(counts = list(classify_complexity(sentence_text))) |>
    dplyr::mutate(HC = counts$HC, LC = counts$LC) |>
    dplyr::ungroup() |>
    dplyr::select(-counts)

  if (!bootstrap) {
    return(tibble::tibble(
      HC = sum(results$HC),
      LC = sum(results$LC)
    ))
  } else {
    boot_counts <- replicate(B, {
      idx <- sample.int(nrow(results), replace = TRUE)
      sampled <- results[idx, ]
      c(HC = sum(sampled$HC), LC = sum(sampled$LC))
    }, simplify = TRUE)

    boot_df <- as.data.frame(t(boot_counts))
    return(tibble::tibble(
      meanHC = mean(boot_df$HC), varHC = stats::var(boot_df$HC),
      meanLC = mean(boot_df$LC), varLC = stats::var(boot_df$LC)
    ))
  }
}

test_that("get_power returns correct structure", {
  skip_if_not(
    tryCatch({spacyr::spacy_initialize(); TRUE}, error = function(e) FALSE),
    "spaCy not available"
  )

  result <- get_power("Malawi", "We will help to improve relations.")
  expect_s3_class(result, "tbl_df")
  expect_true(all(c("P", "OP") %in% names(result)))
})

test_that("get_complex returns correct structure", {
  skip_if_not(
    tryCatch({spacyr::spacy_initialize(); TRUE}, error = function(e) FALSE),
    "spaCy not available"
  )

  result <- get_complex("We should perhaps consider this option.")
  expect_s3_class(result, "tbl_df")
  expect_true(all(c("HC", "LC") %in% names(result)))
})

#' Build entities corpus for entity matching
#'
#' Constructs a vector of lowercased international organization names and
#' country names (including aliases) for use in entity detection.
#'
#' @return A character vector of entity names.
#' @keywords internal
build_entities_corpus <- function() {
  country_list <- countries::list_countries(nomenclature = "name_en")
  country_list <- unique(tolower(country_list))
  country_list <- c(country_list, "ussr", "east germany",
                    "west germany", "soviet union", "the soviet union")
  country_list <- unlist(lapply(country_list, expand_aliases_country))
  c(tolower(io), country_list)
}

#' Build country list for entity matching
#'
#' Constructs a vector of lowercased country names including aliases,
#' without international organizations.
#'
#' @return A character vector of country names.
#' @keywords internal
build_country_list <- function() {
  country_list <- countries::list_countries(nomenclature = "name_en")
  country_list <- unique(tolower(country_list))
  country_list <- c(country_list, "ussr", "east germany",
                    "west germany", "soviet union", "the soviet union")
  unlist(lapply(country_list, expand_aliases_country))
}

#' Classify LTA Traits into Hermann's Leadership Typology
#'
#' Takes per-speech LTA output (from [get_lta()]) and aggregates trait scores
#' across speeches, then classifies the leader along three dimensions
#' (constraint, openness, motivation toward world) and maps the first two plus
#' task orientation to one of eight leadership styles.
#'
#' @param lta A data frame with one row per speech, as returned by
#'   [get_lta()]. Must contain columns `Pp`, `B`, `C`, `Ss`, `Ta`, `D`, `Na`.
#'   When `precision_weighted = TRUE`, must also contain `varPp`, `varB`,
#'   `varC`, `varSs`, `varTa`, `varD`, `varNa`.
#' @param precision_weighted Logical. If `FALSE` (default), aggregate traits
#'   with a simple mean. If `TRUE`, use inverse-variance (precision) weighting
#'   via random-effects meta-analysis (`metafor::rma()`).
#' @param need_for_power Threshold for the need-for-power trait (`Pp`).
#'   Default `0.50`.
#' @param control Threshold for belief in ability to control events (`B`).
#'   Default `0.44`.
#' @param complex_high Threshold for high conceptual complexity (`C`).
#'   Default `0.56`.
#' @param confidence_high Threshold for high self-confidence (`Ss`).
#'   Default `0.81`.
#' @param task Threshold for task orientation (`Ta`). Default `0.59`.
#' @param distrust Threshold for distrust (`D`). Default `0.41`.
#' @param ingroup Threshold for in-group bias / nationalism (`Na`).
#'   Default `0.42`.
#' @return A one-row [tibble::tibble] with aggregated trait values, standard
#'   errors (when `precision_weighted = TRUE`), and classification columns:
#'   `constraint`, `openness`, `motivation_toward_world`, `task_orientation`,
#'   `typology`, and `method`.
#' @export
#' @examples
#' # Simple-mean aggregation works on any data frame with the seven trait
#' # columns; here we use a small illustrative input.
#' example_lta <- data.frame(
#'   Pp = c(0.45, 0.52, 0.48),
#'   B  = c(0.40, 0.48, 0.43),
#'   C  = c(0.60, 0.55, 0.58),
#'   Ss = c(0.75, 0.82, 0.79),
#'   Ta = c(0.62, 0.58, 0.65),
#'   D  = c(0.35, 0.42, 0.38),
#'   Na = c(0.38, 0.45, 0.41)
#' )
#' type_lta(example_lta)
#'
#' \donttest{
#' # Precision-weighted aggregation needs per-trait variances, which come
#' # from get_lta(..., bootstrap = TRUE). Requires spaCy to be installed.
#' spacyr::spacy_initialize()
#' res <- data.table::rbindlist(
#'   lapply(c(jfk19610120, jfk19610925, jfk19630610), function(x)
#'     get_lta(own_entity = "United States", text = clean_text(x),
#'             bootstrap = TRUE, B = 1000))
#' )
#' type_lta(res, precision_weighted = TRUE)
#' }
type_lta <- function(lta,
                     precision_weighted = FALSE,
                     need_for_power = 0.50,
                     control = 0.44,
                     complex_high = 0.56,
                     confidence_high = 0.81,
                     task = 0.59,
                     distrust = 0.41,
                     ingroup = 0.42) {

  traits <- c("Pp", "B", "C", "Ss", "Ta", "D", "Na")

  ## --- Aggregation ---
  if (precision_weighted) {
    if (!requireNamespace("metafor", quietly = TRUE)) {
      stop("Package 'metafor' is required for precision weighting. ",
           "Install it with install.packages('metafor').", call. = FALSE)
    }

    var_traits <- paste0("var", traits)
    missing_vars <- setdiff(var_traits, names(lta))
    if (length(missing_vars) > 0) {
      stop("Columns missing for precision weighting: ",
           paste(missing_vars, collapse = ", "),
           ". Run get_lta() with bootstrap = TRUE.", call. = FALSE)
    }

    estimates <- vapply(seq_along(traits), function(i) {
      yi <- lta[[traits[i]]]
      vi <- lta[[var_traits[i]]]
      vi <- pmax(vi, 1e-8, na.rm = TRUE)
      fit <- metafor::rma(yi = yi, vi = vi, method = "REML")
      c(mu = as.numeric(fit$b), se = as.numeric(fit$se))
    }, numeric(2))

    vals <- stats::setNames(estimates[1, ], traits)
    ses  <- stats::setNames(estimates[2, ], paste0("se_", traits))
    out  <- tibble::as_tibble(c(as.list(vals), as.list(ses)))

  } else {
    vals <- vapply(traits, function(tr) mean(lta[[tr]], na.rm = TRUE), numeric(1))
    out  <- tibble::as_tibble(as.list(vals))
  }

  ## --- Classification ---
  out$constraint <- ifelse(
    vals["Pp"] < need_for_power & vals["B"] < control,
    "Respect", "Challenge"
  )

  out$openness <- ifelse(
    vals["C"] > vals["Ss"] |
      (vals["C"] > complex_high & vals["Ss"] > confidence_high),
    "Open", "Closed"
  )

  out$motivation_toward_world <- ifelse(
    vals["D"] < distrust & vals["Na"] < ingroup, "Cooperative",
    ifelse(vals["D"] < distrust & vals["Na"] >= ingroup,
           "Cooperative (in-group bias)",
           ifelse(vals["D"] >= distrust & vals["Na"] < ingroup,
                  "Competitive (out-group focus)", "Competitive"))
  )

  out$task_orientation <- ifelse(vals["Ta"] > task, "Problem", "Relationship")

  ## --- Typology mapping ---
  typology_map <- list(
    Challenge = list(
      Closed = list(Problem = "Expansionistic", Relationship = "Evangelistic"),
      Open   = list(Problem = "Incremental",    Relationship = "Charismatic")
    ),
    Respect = list(
      Closed = list(Problem = "Directive",  Relationship = "Consultative"),
      Open   = list(Problem = "Reactive",   Relationship = "Accommodative")
    )
  )

  out$typology <- typology_map[[out$constraint]][[out$openness]][[out$task_orientation]]
  out$method <- ifelse(precision_weighted, "precision_weighted", "simple_mean")

  out
}

#' Compute task orientation scores
#'
#' Classifies lemmas in the parsed text as task-instrumental (TI) or
#' interpersonal (IP) based on codebook word lists, with token-level
#' quote and negation handling.
#'
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{TI} and \code{IP}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanTI}, \code{meanIP}, \code{varTI}, \code{varIP}.
#' @export
get_task <- function(text, bootstrap = FALSE, B = 1000) {

  task_words <- tolower(c(
    "accomplishment", "achievement", "attainment", "achieve", "realize", "surmount",
    "action", "activity", "advice", "advise", "proposal", "recommendation",
    "applied", "application", "arrangement", "assignment", "task", "attempt",
    "endeavor", "strive", "try", "build", "built", "case", "issue", "problem",
    "question", "carry", "pursue", "change", "construct", "decision",
    "development", "progress", "difficulty", "direction", "supervision",
    "efficiency", "effort", "fight", "function", "implementation", "implementing",
    "improve", "improvement", "intervene", "intervention", "inform", "information",
    "initiate", "initiative", "inquiry", "investigation", "study", "experiment",
    "investment", "know-how", "specialization", "training", "measure", "step",
    "tactic", "way", "means", "mistake", "organize", "organization", "output",
    "product", "result", "pay", "payment", "repayment", "pledge", "position",
    "prepare", "preparation", "priority", "procedure", "process", "production",
    "recruit", "recruiting", "recruitment", "report", "resolution", "solution",
    "stage", "start", "struggle", "success", "succeed", "fail", "failure",
    "timetable", "toil", "work", "useful", "veto", "aim", "aspiration", "cause",
    "end", "goal", "objective", "plan", "platform", "policy", "program", "project",
    "purpose", "strategy", "target"
  ))

  affect_words <- tolower(c(
    "amnesty", "appreciate", "appreciation", "assist", "assistance", "assure",
    "assurance", "reassure", "reassurance", "award", "reward", "beneficial",
    "brother", "brothers and sisters", "calm", "care", "celebrate", "celebration",
    "collaboration", "comfort", "comrade", "comradely", "concern", "confident",
    "confidence", "congratulations", "content", "cooperation", "coordination",
    "deserve", "dignity", "equality", "faith", "forgive", "forgiveness",
    "fraternal", "fraternity", "freedom", "friendship", "grateful", "gratitude",
    "hand in hand", "happiness", "help", "honor", "hope", "hospitality",
    "human rights", "independence", "innocent", "justice", "liberation", "love",
    "loyalty", "paradise", "utopia", "patience", "pay respects", "peace",
    "pleasure", "popular", "popularity", "pride", "reconciliation", "safety",
    "security", "serve", "service", "sincere", "solidarity", "support",
    "sympathy", "trust", "understanding", "unity", "united", "welcome",
    "welfare", "well-being"
  ))

  parsed <- spacyr::spacy_parse(text, dependency = TRUE, lemma = TRUE)

  # FIX: Token-level classification instead of sentence-level skipping
  classify_sentence <- function(df) {
    # FIX 1: Track quote spans - skip tokens WITHIN quotes, not entire sentence
    df$is_quote_char <- df$token %in% c('"', "'", "\u201C", "\u201D", "\u2018", "\u2019")
    df$quote_cumsum <- cumsum(df$is_quote_char)
    df$in_quote <- (df$quote_cumsum %% 2 == 1)

    # FIX 2: Track negation scope - skip tokens under negation, not entire sentence
    negation_heads <- df$head_token_id[df$dep_rel == "neg"]
    df$is_negated <- df$token_id %in% negation_heads

    result <- character(0)

    for (i in seq_len(nrow(df))) {
      row <- df[i, ]
      lemma <- tolower(row$lemma)

      # Skip if in quote or negated
      if (row$in_quote || row$is_negated) next

      if (lemma %in% task_words) result <- c(result, "TI")
      else if (lemma %in% affect_words) result <- c(result, "IP")
    }

    result

  }

  classify_all <- function(parsed_df) {

    parsed |>
      dplyr::group_by(sentence_id) |>
      dplyr::summarise(
        tags = list(classify_sentence(dplyr::pick(dplyr::everything()))),
        TI = sum(unlist(tags) == "TI"),
        IP = sum(unlist(tags) == "IP")
      ) |>
      dplyr::select(-tags)

  }

  results <- classify_all(parsed)

  if (length(results) == 0) {

    if (bootstrap) {

      return(tibble::tibble(meanTI = 0, meanIP = 0, varTI = 0, varIP = 0))

    } else {

      return(tibble::tibble(TI = 0, IP = 0))

    }

  } else {

    if (bootstrap) {

      boot_counts <- replicate(B, {

        sampled_id <- unique(parsed$sentence_id)[sample(length(unique(parsed$sentence_id)), replace = TRUE)]
        sample_df <- tibble::tibble(sentence_id = sampled_id)

        own_sentences_sampled <- sample_df |>
          dplyr::inner_join(results, by = "sentence_id", relationship = "many-to-many")

        c(TI = sum(own_sentences_sampled$TI),
          IP = sum(own_sentences_sampled$IP))

      }, simplify = TRUE)

      boot_df <- as.data.frame(t(boot_counts))
      return(tibble::tibble(
        meanTI = mean(boot_df$TI), varTI = stats::var(boot_df$TI),
        meanIP = mean(boot_df$IP), varIP = stats::var(boot_df$IP)
      ))

    } else {

      return(tibble::tibble(TI = sum(results$TI), IP = sum(results$IP)))

    }

  }

}

#' Compute affiliation scores
#'
#' Measures the affiliation orientation of a leader's speech by classifying
#' verb-level actions as affiliative (A) or other-affiliative (OA) based on
#' the speaker's own-entity references and sentence-level context.
#'
#' @param own_entity A character vector of entity names representing the
#'   speaker's own country or group.
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
#'   columns are \code{A} and \code{OA}. When \code{bootstrap = TRUE}, columns
#'   are \code{meanA}, \code{meanOA}, \code{varA}, \code{varOA}.
#' @export
get_aff <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  # Build entities corpus

  all_entities_corpus <- build_entities_corpus()

  # Regex
  regex_a <- stringr::regex(
    "\\b(friendship|friend(s)?|sympathy|support|cooperate|cooperation|unity|peaceful|amicable|cordial|alliance|ally|solidarity|harmony|collaboration|collaborate|partnership|camaraderie|goodwill|forgive|accept|welcome)\\b",
    ignore_case = TRUE)
  regex_b <- stringr::regex(
    "\\b(agree|agreement|resolve|negotiat|dialog(ue)?|talk(s)?|compromise|concile|restore|reconcile|rapprochement|bridge|resume|resum|reunite|settle|mend|regret)\\b",
    ignore_case = TRUE)
  regex_c <- stringr::regex(
    "\\b(summit|conference|forum|visit|meeting|dialogue|dialog|talk(s)?|joint|coalition|collaborate|together|program)\\b",
    ignore_case = TRUE)
  regex_d <- stringr::regex(
    "\\b(help|aid|assist|support|relief|care|comfort|console|concern|concerned|compassion|charit|nurtur|alleviat|suffer|plight|victim|refugee)\\b",
    ignore_case = TRUE)

  ## Define own and other entities
  own_terms <- c(unique(tolower(own_entity)),
                 "we", "our nation", "our country", "us",
                 "our state", "our government", "our institution",
                 "our group", "our party", "our people",
                 "i", "my nation", "my country", "me",
                 "my state", "my government", "my institution",
                 "my group", "my party", "my people")

  own_entity <- unique(c(tolower(own_entity), expand_aliases_country(own_entity)))

  ## Define own and other entities
  other_terms <- c(setdiff(all_entities_corpus, tolower(own_terms)),
                   "they", "their nation", "their country",
                   "their state", "their government", "their institution")

  # Parse text
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)

  sentences <- parsed |>
    dplyr::group_by(sentence_id) |>
    dplyr::summarise(sentence = paste(token, collapse = " "), .groups = "drop")

  # Step 1: Get multiword named entities
  parsed_entity_grouped <- parsed |>
    dplyr::mutate(ent_group = ifelse(!is.na(entity) & entity != "", 1, NA)) |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(entity_group_id = data.table::rleid(ent_group)) |>
    dplyr::ungroup() |>
    dplyr::mutate(entity_group_id = ifelse(is.na(ent_group), NA, entity_group_id))

  multiword_entities <- parsed_entity_grouped |>
    dplyr::filter(!is.na(entity_group_id)) |>
    dplyr::group_by(doc_id, sentence_id, entity_group_id) |>
    dplyr::summarise(entity = paste(token, collapse = " "), .groups = "drop")

  # Step 2: Get subjects and join with named entities (new, to incorporate possessives)
  subjects <- parsed_entity_grouped |>
    dplyr::filter(dep_rel %in% c("poss", "nsubj", "nsubjpass")) |>
    dplyr::select(-entity) |>
    dplyr::left_join(multiword_entities, by = c("doc_id", "sentence_id", "entity_group_id")) |>
    dplyr::mutate(
      poss_pronoun = dplyr::lag(lemma),
      poss_dep = dplyr::lag(dep_rel),
      has_possessive = poss_pronoun %in% c("my", "our", "their") & poss_dep == "poss",
      subject_phrase = dplyr::case_when(
        has_possessive ~ paste(poss_pronoun, lemma),        # e.g., "our concern"
        !is.na(entity) ~ entity,                             # multiword entities
        TRUE ~ token
      )
    ) |>
    dplyr::filter(dep_rel %in% c("nsubj", "nsubjpass") | (dep_rel == "poss" & has_possessive)) |>
    dplyr::select(doc_id, sentence_id, subject_phrase, head_token_id) |>
    dplyr::distinct() |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::summarise(subject = tolower(dplyr::first(subject_phrase)),
              head_token_id = dplyr::first(head_token_id), .groups = "drop")

  sentences <- parsed |>
    dplyr::group_by(sentence_id) |>
    dplyr::summarise(sentence = paste(token, collapse = " "), .groups = "drop") |>
    dplyr::left_join(subjects, by = "sentence_id") |>
    dplyr::mutate(entity_type = ifelse(subject %in% own_terms, "own", NA_character_))

  ## Passive voice: Handle passive voice: "by us" etc. -----
  agent_tokens <- parsed |>
    dplyr::filter(dep_rel == "agent") |>
    dplyr::select(doc_id, sentence_id, agent_token_id = token_id)

  agent_objects <- parsed |>
    dplyr::filter(dep_rel == "pobj" & tolower(token) %in% c("us", "we", "our", "my")) |>
    dplyr::rename(child_token_id = token_id, agent_actor = token)

  agents <- agent_tokens |>
    dplyr::left_join(agent_objects,
              by = c("doc_id", "sentence_id", "agent_token_id" = "head_token_id")) |>
    dplyr::filter(!is.na(agent_actor)) |>
    dplyr::select(doc_id, sentence_id, agent_actor)

  sentences_with_agents <- sentences |>
    dplyr::left_join(agents, by = "sentence_id")

  ## Identify sentences with own subjects or own agents -----
  own_terms_regex <- paste0(stringr::str_replace_all(tolower(own_terms), "([\\W])", "\\\\\\1"),
                            collapse = "|")

  sentences_with_agents <- sentences_with_agents |>
    dplyr::mutate(
      subject_norm   = subject,
      agent_norm     = stringr::str_to_lower(agent_actor),
      starts_with_own_pron = stringr::str_detect(subject_norm, "^(we|us|our|my)\\b"),
      starts_with_own_name = if (nchar(own_terms_regex) > 0)
        stringr::str_detect(subject_norm, paste0("^(?:", own_terms_regex, ")\\b")) else FALSE,
      exact_own_match = subject_norm %in% tolower(own_terms)
    )

  own_sentences <- sentences_with_agents |>
    dplyr::filter(
      exact_own_match | starts_with_own_pron | starts_with_own_name |
        agent_norm %in% tolower(own_terms)
    )

  verbs <- parsed |>
    dplyr::filter(pos == "VERB") |>
    dplyr::inner_join(
      own_sentences |> dplyr::select(sentence_id, head_token_id),
      by = c("sentence_id", "token_id" = "head_token_id")
    )

  classify_aff <- function(lemma, sentence) {
    lemma <- tolower(lemma)
    sentence <- tolower(sentence)

    if (lemma %in% c("will", "would", "shall", "should", "can", "could", "may", "might",
                     "must", "do", "does", "did", "have", "has", "had",
                     "be", "am", "is", "are", "was", "were", "being", "been")) {
      return(NA_character_)
    }

    cond_a <- stringr::str_detect(sentence, regex_a)
    cond_b <- stringr::str_detect(sentence, regex_b)
    cond_c <- stringr::str_detect(sentence, regex_c)
    cond_d <- stringr::str_detect(sentence, regex_d)

    if (cond_a | cond_b | cond_c | cond_d) {
      return("A")
    } else {
      return("OA")
    }
  }

  if (nrow(own_sentences) == 0 | nrow(verbs) == 0) {
    if (bootstrap) {
      return(tibble::tibble(meanA = 0, meanOA = 0, varA = 0, varOA = 0))
    } else {
      return(tibble::tibble(A = 0, OA = 0))
    }
  }

  verbs_joined <- verbs |>
    dplyr::left_join(own_sentences, by = "sentence_id") |>
    dplyr::filter(!is.na(sentence))

  verb_classified <- verbs_joined |>
    dplyr::rowwise() |>
    dplyr::mutate(
      affiliation = classify_aff(lemma, sentence)
    ) |>
    dplyr::ungroup() |>
    dplyr::filter(!is.na(affiliation))

  output <- verb_classified |>
    dplyr::count(affiliation) |>
    tidyr::complete(affiliation = c("A", "OA"), fill = list(n = 0)) |>
    tidyr::pivot_wider(names_from = affiliation, values_from = n)

  if (bootstrap) {

    if (nrow(verb_classified) == 0) {

      btres <- tibble::tibble(meanA = 0, meanOA = 0, varA = 0, varOA = 0)

    } else {

    sentence_summary <- verb_classified |>
      dplyr::group_by(sentence_id) |>
      dplyr::count(affiliation) |>
      tidyr::complete(affiliation = c("A", "OA"), fill = list(n = 0)) |>
      tidyr::pivot_wider(names_from = affiliation, values_from = n)

    boot_results <- purrr::map_dfr(1:B, ~{
      sampled_id <- sentences$sentence_id[sample(nrow(sentences), replace = TRUE)]
      sample_df <- tibble::tibble(sentence_id = sampled_id)

      own_sentences_sampled <- sample_df |>
        dplyr::inner_join(sentence_summary, by = "sentence_id", relationship = "many-to-many")

      if (nrow(own_sentences_sampled) == 0) {

        tibble::tibble(A = 0, OA = 0)

      } else {

        own_sentences_sampled |>
          dplyr::summarise(
            A = sum(A),
            OA = sum(OA)
          )

      }
    })

    btres <- boot_results |>
                   dplyr::summarise(meanA = mean(A), meanOA = mean(OA),
                             varA = stats::var(A), varOA = stats::var(OA))

    }

    return(btres)

  } else {

    return(output)

  }
}

#' Leadership Trait Analysis (LTA)
#'
#' Runs all eight LTA trait functions at once and returns a single combined
#' tibble. The eight traits are: power, affiliation, distrust, conceptual
#' complexity, task orientation, self-confidence, nationalism, and control.
#'
#' @param own_entity A character string identifying the speaker's country or
#'   entity (e.g., `"United States"`).
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical. If `TRUE`, returns bootstrap means and variances
#'   instead of raw counts. Default is `FALSE`.
#' @param B Number of bootstrap iterations. Default is `1000`.
#' @return A one-row [tibble::tibble].
#'
#'   When `bootstrap = FALSE`, columns include raw counts (`P`, `OP`, `A`,
#'   `OA`, `S`, `OS`, `HC`, `LC`, `TI`, `IP`, `SC`, `OSC`, `N`, `ON`, `IC`,
#'   `OC`) plus trait proportions (`Pp`, `D`, `C`, `Ta`, `Ss`, `Na`, `B`).
#'
#'   When `bootstrap = TRUE`, columns include bootstrap means and variances
#'   for the raw counts (`meanP`, `varP`, `meanOP`, `varOP`, ...) plus trait
#'   proportions and their delta-method variances (`Pp`, `varPp`, `D`, `varD`,
#'   `C`, `varC`, `Ta`, `varTa`, `Ss`, `varSs`, `Na`, `varNa`, `B`, `varB`).
#' @export
#' @examples
#' \donttest{
#' # Requires spaCy to be installed; see spacyr::spacy_install().
#' spacyr::spacy_initialize()
#' get_lta(own_entity = "United States", text = "We will defend our nation.")
#' get_lta(own_entity = "United States", text = "We will defend our nation.",
#'         bootstrap = TRUE, B = 500)
#' }
get_lta <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  res_power   <- get_power(own_entity = own_entity, text = text,
                           bootstrap = bootstrap, B = B)
  res_aff     <- get_aff(own_entity = own_entity, text = text,
                         bootstrap = bootstrap, B = B)
  res_dist    <- get_dist(own_entity = own_entity, text = text,
                          bootstrap = bootstrap, B = B)
  res_complex <- get_complex(text = text, bootstrap = bootstrap, B = B)
  res_task    <- get_task(text = text, bootstrap = bootstrap, B = B)
  res_conf    <- get_conf(text = text, bootstrap = bootstrap, B = B)
  res_nat     <- get_nat(own_entity = own_entity, text = text,
                         bootstrap = bootstrap, B = B)
  res_ctrl    <- get_ctrl(own_entity = own_entity, text = text,
                          bootstrap = bootstrap, B = B)

  dat_out <- dplyr::bind_cols(
    res_power, res_aff, res_dist, res_complex, res_task, res_conf, res_nat,
    res_ctrl
  )

  if (bootstrap) {
    dat_out <- dat_out |>
      mutate(
        Pp = if_else((meanP + meanOP) == 0, 0,
                     meanP / (meanP + meanOP)),
        varPp = if_else((meanP + meanOP) == 0, 0,
                        ((meanOP / (meanP + meanOP)^2)^2) * varP +
                          ((-meanP / (meanP + meanOP)^2)^2) * varOP),

        D = if_else((meanS + meanOS) == 0, 0,
                    meanS / (meanS + meanOS)),
        varD = if_else((meanS + meanOS) == 0, 0,
                       ((meanOS / (meanS + meanOS)^2)^2) * varS +
                         ((-meanS / (meanS + meanOS)^2)^2) * varOS),

        C = if_else((meanHC + meanLC) == 0, 0,
                    meanHC / (meanHC + meanLC)),
        varC = if_else((meanHC + meanLC) == 0, 0,
                       ((meanLC / (meanHC + meanLC)^2)^2) * varHC +
                         ((-meanHC / (meanHC + meanLC)^2)^2) * varLC),

        Ta = if_else((meanTI + meanIP) == 0, 0,
                     meanTI / (meanTI + meanIP)),
        varTa = if_else((meanTI + meanIP) == 0, 0,
                        ((meanIP / (meanTI + meanIP)^2)^2) * varTI +
                          ((-meanTI / (meanTI + meanIP)^2)^2) * varIP),

        Ss = if_else((meanSC + meanOSC) == 0, 0,
                     meanSC / (meanSC + meanOSC)),
        varSs = if_else((meanSC + meanOSC) == 0, 0,
                        ((meanOSC / (meanSC + meanOSC)^2)^2) * varSC +
                          ((-meanSC / (meanSC + meanOSC)^2)^2) * varOSC),

        Na = if_else((meanN + meanON) == 0, 0,
                     meanN / (meanN + meanON)),
        varNa = if_else((meanN + meanON) == 0, 0,
                        ((meanON / (meanN + meanON)^2)^2) * varN +
                          ((-meanN / (meanN + meanON)^2)^2) * varON),

        B = if_else((meanIC + meanOC) == 0, 0,
                    meanIC / (meanIC + meanOC)),
        varB = if_else((meanIC + meanOC) == 0, 0,
                       ((meanOC / (meanIC + meanOC)^2)^2) * varIC +
                         ((-meanIC / (meanIC + meanOC)^2)^2) * varOC)
      )
  } else {
    dat_out <- dat_out |>
      mutate(
        Pp = if_else((P + OP) == 0, 0, P / (P + OP)),
        D  = if_else((S + OS) == 0, 0, S / (S + OS)),
        C  = if_else((HC + LC) == 0, 0, HC / (HC + LC)),
        Ta = if_else((TI + IP) == 0, 0, TI / (TI + IP)),
        Ss = if_else((SC + OSC) == 0, 0, SC / (SC + OSC)),
        Na = if_else((N + ON) == 0, 0, N / (N + ON)),
        B  = if_else((IC + OC) == 0, 0, IC / (IC + OC))
      )
  }

  dat_out
}

# Regex patterns for power conditions
regex_a <- stringr::regex("\\b(wipe( them)? out|defeat(ed)?|destroy(ed)?|attack(ed)?|assault(ed)?|threat(en|ened)?|accus(ed|e)?|reprimand(ed)?|retaliat(ed|e)?|blame(d)?)\\b", ignore_case = TRUE)
regex_b <- stringr::regex("\\b(help(ed)?|assist(ed)?|support(ed)?|aid(ed)?|encourag(ed|e)?|advise(d)?)\\b.*\\b(them|another|others|recipient|nation(s)?|group(s)?)\\b", ignore_case = TRUE)
regex_c <- stringr::regex("\\b(arrang(ed)?|regulat(ed|e)?|control(led)?|determin(e|ed)?|check(ed)?|investigat(ed|e)?|search(ed)?|monitor(ed)?)\\b.*\\b(them|their|another|lives|actions?)\\b", ignore_case = TRUE)
regex_d <- stringr::regex("\\b(influenc(e|ed)?|persuad(e|ed)?|convinc(e|ed)?|brib(e|ed)?|argu(e|ed)?|suggest(ed)?|hint(ed)?|pressure(d)?|advocat(e|ed)?|hope(d)?)\\b.*\\b(them|recipient|group|nation|he|she|you)\\b", ignore_case = TRUE)
regex_e <- stringr::regex("\\b(impress(ed)?|prestig(e|ious)?|display(ed)?|show(ed|case(d)?)?|notoriety|fame|glory|take|took|taken|honor(ed)?)\\b", ignore_case = TRUE)
regex_f <- stringr::regex("\\b(vindicat(ed)?|superior|status|reputation|weak(ness)?|inferior|strength|power(ful)?|prestige|blame(d)?|emphasiz(e|ed)|criticized|guarantee(d)?)\\b", ignore_case = TRUE)

p_keywords <- c(
  # Original codebook list (from LTA codebook line 47)
  "warn", "help", "hint", "stop", "suggest", "impress", "promise", "worried",
  "attack", "demand", "refuse", "accuse", "protect", "defend", "retaliate", "blame",
  "support", "threaten", "guard", "achieve", "instigate", "arrange", "hope",
  # From codebook examples
  "take",       # Example 5: "If we take this action" → P (condition e)
  "emphasize"   # Example 6: "I emphasize the significance" → P (condition f)
)

#' Compute Power Score (LTA)
#'
#' Classifies verbs in text as reflecting Power (P) or Other Power (OP)
#' based on the Leadership Trait Analysis codebook.
#'
#' @param own_entity Character vector of the speaker's country/entity name(s).
#' @param text Character string of the speech text to analyse.
#' @param bootstrap Logical; if TRUE, return bootstrap means and variances.
#' @param B Number of bootstrap replications (default 1000).
#' @return A tibble with columns P and OP (or meanP, meanOP, varP, varOP if
#'   bootstrap = TRUE).
#' @export
get_power <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  # Own / other terms
  own_terms <- c(
    unique(tolower(own_entity)),
    "we", "our nation", "our country", "us",
    "our state", "our government", "our institution",
    "our group", "our party", "our people",
    "i", "my nation", "my country", "me",
    "my state", "my government", "my institution",
    "my group", "my party", "my people"
  )

  # expand aliases if you use your helper
  own_entity <- unique(c(tolower(own_entity), expand_aliases_country(own_entity)))
  own_terms  <- unique(c(tolower(own_terms), own_entity))
  own_terms_norm <- stringr::str_to_lower(stringr::str_replace(own_terms, "^the\\s+", ""))

  # Parse
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)

  if (nrow(parsed) == 0) {
    if (bootstrap) return(tibble::tibble(meanP = 0, meanOP = 0, varP = 0, varOP = 0))
    return(tibble::tibble(P = 0, OP = 0))
  }

  # Sentence strings
  sentences <- parsed |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::summarise(sentence = paste(token, collapse = " "), .groups = "drop")

  # Multiword entity grouping (contiguous entity tokens)
  parsed_entity_grouped <- parsed |>
    dplyr::mutate(ent_group = ifelse(!is.na(entity) & entity != "", 1L, NA_integer_)) |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(entity_group_id = data.table::rleid(ent_group)) |>
    dplyr::ungroup() |>
    dplyr::mutate(entity_group_id = ifelse(is.na(ent_group), NA_integer_, entity_group_id))

  entity_groups <- parsed_entity_grouped |>
    dplyr::filter(!is.na(entity_group_id)) |>
    dplyr::group_by(doc_id, sentence_id, entity_group_id) |>
    dplyr::summarise(entity_phrase = paste(token, collapse = " "), .groups = "drop")

  # token_id -> entity_group_id map (for quick lookup)
  token_to_entity_group <- parsed_entity_grouped |>
    dplyr::filter(!is.na(entity_group_id)) |>
    dplyr::select(doc_id, sentence_id, token_id, entity_group_id) |>
    dplyr::distinct()

  # Possessive map: poss -> head noun (e.g., our -> government)
  poss_map <- parsed |>
    dplyr::filter(dep_rel == "poss") |>
    dplyr::transmute(
      doc_id, sentence_id,
      head_noun_id = head_token_id,               # noun token_id
      poss_lemma   = stringr::str_to_lower(lemma) # "our"/"my"/"their"
    ) |>
    dplyr::filter(poss_lemma %in% c("my", "our", "their")) |>
    dplyr::distinct()

  # Helper: build phrase for an actor token_id in a sentence
  build_actor_phrase <- function(doc_id_, sentence_id_, actor_token_id_, parsed_df) {
    # Force evaluation of parameters
    doc_id_ <- doc_id_

    sentence_id_ <- sentence_id_
    actor_token_id_ <- actor_token_id_

    # 1) if part of a multiword entity, return the whole entity phrase
    eg_rows <- token_to_entity_group[
      token_to_entity_group$doc_id == doc_id_ &
      token_to_entity_group$sentence_id == sentence_id_ &
      token_to_entity_group$token_id == actor_token_id_, ]

    if (nrow(eg_rows) == 1) {
      eg <- eg_rows$entity_group_id[1]
      if (!is.na(eg)) {
        ent_rows <- entity_groups[
          entity_groups$doc_id == doc_id_ &
          entity_groups$sentence_id == sentence_id_ &
          entity_groups$entity_group_id == eg, ]
        if (nrow(ent_rows) == 1 && !is.na(ent_rows$entity_phrase[1])) {
          return(stringr::str_to_lower(ent_rows$entity_phrase[1]))
        }
      }
    }

    # 2) else: base token
    tok <- parsed_df[
      parsed_df$doc_id == doc_id_ &
      parsed_df$sentence_id == sentence_id_ &
      parsed_df$token_id == actor_token_id_, ]

    if (nrow(tok) == 0) return(NA_character_)
    base <- stringr::str_to_lower(tok$token[1])

    # 3) if noun has possessive ("our X"), prepend it
    poss_rows <- poss_map[
      poss_map$doc_id == doc_id_ &
      poss_map$sentence_id == sentence_id_ &
      poss_map$head_noun_id == actor_token_id_, ]

    if (nrow(poss_rows) >= 1 && !is.na(poss_rows$poss_lemma[1])) {
      return(paste(poss_rows$poss_lemma[1], base))
    }

    return(base)
  }

  # Build verb-level actor table
  # (A) Active + passive subjects: nsubj/nsubjpass point to VERB via head_token_id
  verb_subjects <- parsed |>
    dplyr::filter(dep_rel %in% c("nsubj", "nsubjpass")) |>
    dplyr::transmute(
      doc_id, sentence_id,
      verb_id  = head_token_id,   # verb token_id
      actor_id = token_id
    ) |>
    dplyr::distinct()

  # (B) Passive agents: agent token ("by") attached to VERB; pobj under that agent is the actor
  by_tokens <- parsed |>
    dplyr::filter(dep_rel == "agent") |>
    dplyr::transmute(
      doc_id, sentence_id,
      verb_id = head_token_id,  # verb token_id
      by_id   = token_id
    ) |>
    dplyr::distinct()

  agent_pobj <- parsed |>
    dplyr::filter(dep_rel == "pobj") |>
    dplyr::transmute(
      doc_id, sentence_id,
      by_id     = head_token_id,
      actor_id  = token_id
    ) |>
    dplyr::distinct()

  verb_agents <- by_tokens |>
    dplyr::inner_join(agent_pobj, by = c("doc_id", "sentence_id", "by_id")) |>
    dplyr::select(doc_id, sentence_id, verb_id, actor_id) |>
    dplyr::distinct()

  verb_actors <- dplyr::bind_rows(verb_subjects, verb_agents) |>
    dplyr::distinct()

  # Propagate subjects through auxiliary chains
  aux_chains <- parsed |>
    dplyr::filter(dep_rel == "aux") |>
    dplyr::transmute(
      doc_id, sentence_id,
      aux_id = token_id,           # the modal/aux token
      main_verb_id = head_token_id  # the main verb it modifies
    )

  # For each main verb, find if its aux has a subject
  aux_subjects <- aux_chains |>
    dplyr::inner_join(
      verb_actors |> dplyr::select(doc_id, sentence_id, verb_id, actor_id),
      by = c("doc_id" = "doc_id", "sentence_id" = "sentence_id", "aux_id" = "verb_id")
    ) |>
    dplyr::transmute(
      doc_id, sentence_id,
      verb_id = main_verb_id,  # propagate subject to main verb
      actor_id
    )

  # Combine direct subjects and propagated subjects
  verb_actors <- dplyr::bind_rows(verb_actors, aux_subjects) |>
    dplyr::distinct(doc_id, sentence_id, verb_id, .keep_all = TRUE)

  # Propagate subjects through xcomp (infinitive complements)
  xcomp_chains <- parsed |>
    dplyr::filter(dep_rel == "xcomp") |>
    dplyr::transmute(
      doc_id, sentence_id,
      xcomp_id = token_id,           # the infinitive verb
      parent_verb_id = head_token_id  # the controlling verb
    )

  xcomp_subjects <- xcomp_chains |>
    dplyr::inner_join(
      verb_actors |> dplyr::select(doc_id, sentence_id, verb_id, actor_id),
      by = c("doc_id" = "doc_id", "sentence_id" = "sentence_id", "parent_verb_id" = "verb_id")
    ) |>
    dplyr::transmute(
      doc_id, sentence_id,
      verb_id = xcomp_id,
      actor_id
    )

  verb_actors <- dplyr::bind_rows(verb_actors, xcomp_subjects) |>
    dplyr::distinct(doc_id, sentence_id, verb_id, .keep_all = TRUE)

  # Propagate subjects through conj (coordinated verbs)
  conj_chains <- parsed |>
    dplyr::filter(dep_rel == "conj" & pos %in% c("VERB", "AUX")) |>
    dplyr::transmute(
      doc_id, sentence_id,
      conj_id = token_id,           # the conjunct verb
      head_verb_id = head_token_id  # the head verb
    )

  conj_subjects <- conj_chains |>
    dplyr::inner_join(
      verb_actors |> dplyr::select(doc_id, sentence_id, verb_id, actor_id),
      by = c("doc_id" = "doc_id", "sentence_id" = "sentence_id", "head_verb_id" = "verb_id")
    ) |>
    dplyr::transmute(
      doc_id, sentence_id,
      verb_id = conj_id,
      actor_id
    )

  verb_actors <- dplyr::bind_rows(verb_actors, conj_subjects) |>
    dplyr::distinct(doc_id, sentence_id, verb_id, .keep_all = TRUE)

  # Add actor phrase
  verb_actors <- verb_actors |>
    dplyr::rowwise() |>
    dplyr::mutate(actor = build_actor_phrase(doc_id, sentence_id, actor_id, parsed)) |>
    dplyr::ungroup() |>
    dplyr::mutate(
      actor_norm = stringr::str_to_lower(stringr::str_replace(actor, "^the\\s+", "")),
      is_own =
        actor_norm %in% own_terms_norm |
        stringr::str_detect(actor_norm, "^(we|us|our|my)\\b")
    ) |>
    dplyr::filter(is_own) |>
    dplyr::select(doc_id, sentence_id, verb_id, actor, actor_norm)

  if (nrow(verb_actors) == 0) {
    if (bootstrap) return(tibble::tibble(meanP = 0, meanOP = 0, varP = 0, varOP = 0))
    return(tibble::tibble(P = 0, OP = 0))
  }

  # Pull the verbs we will classify (verb_id = token_id)
  verbs <- parsed |>
    dplyr::filter(pos %in% c("VERB", "AUX")) |>
    dplyr::transmute(
      doc_id, sentence_id,
      verb_id = token_id,
      lemma   = stringr::str_to_lower(lemma),
      token   = stringr::str_to_lower(token),
      tag, dep_rel
    ) |>
    dplyr::inner_join(verb_actors, by = c("doc_id", "sentence_id", "verb_id")) |>
    dplyr::inner_join(sentences, by = c("doc_id", "sentence_id"))

  if (nrow(verbs) == 0) {
    if (bootstrap) return(tibble::tibble(meanP = 0, meanOP = 0, varP = 0, varOP = 0))
    return(tibble::tibble(P = 0, OP = 0))
  }

  # Classifier (your logic, but no sentence-level dropping)
  classify_power <- function(lemma, tag, sentence) {
    if (is.na(sentence) || is.na(lemma)) return(NA_character_)

    # drop auxiliaries/modals (keep classification focused on action verbs)
    if (lemma %in% c("will","would","shall","should","can","could","may","might",
                     "must","do","does","did","have","has","had",
                     "be","am","is","are","was","were","being","been")) {
      return(NA_character_)
    }

    # optional: drop bare infinitives like "to help" when it's just complement of "want/seek"
    a <- stringr::str_detect(sentence, regex_a)
    b <- stringr::str_detect(sentence, regex_b)
    c <- stringr::str_detect(sentence, regex_c)
    d <- stringr::str_detect(sentence, regex_d)
    e <- stringr::str_detect(sentence, regex_e)
    f <- stringr::str_detect(sentence, regex_f)

    if (a | b | c | d | e | f | lemma %in% p_keywords) "P" else "OP"
  }

  verb_classified <- verbs |>
    dplyr::rowwise() |>
    dplyr::mutate(power = classify_power(lemma, tag, sentence)) |>
    dplyr::ungroup() |>
    dplyr::filter(!is.na(power))

  # Output
  if (nrow(verb_classified) == 0) {

    if (bootstrap) return(tibble::tibble(meanP = 0, meanOP = 0, varP = 0, varOP = 0))
    return(tibble::tibble(P = 0, OP = 0))

  } else {

    if (!bootstrap) {

      output <- verb_classified |>
        dplyr::count(power) |>
        tidyr::complete(power = c("P","OP"), fill = list(n = 0)) |>
        tidyr::pivot_wider(names_from = power, values_from = n) |>
        dplyr::select(P, OP)

      return(output)

    } else {

      # Bootstrap by sentence resampling
      sentence_summary <- verb_classified |>
        dplyr::group_by(doc_id, sentence_id) |>
        dplyr::count(power) |>
        tidyr::complete(power = c("P","OP"), fill = list(n = 0)) |>
        tidyr::pivot_wider(names_from = power, values_from = n) |>
        dplyr::ungroup() |>
        dplyr::select(doc_id, sentence_id, P, OP)

      all_sentence_ids <- sentences |>
        dplyr::distinct(doc_id, sentence_id)

      boot_results <- purrr::map_dfr(1:B, ~{
        sampled <- all_sentence_ids |>
          dplyr::slice_sample(n = nrow(all_sentence_ids), replace = TRUE)

        sampled |>
          dplyr::inner_join(sentence_summary, by = c("doc_id","sentence_id")) |>
          dplyr::summarise(P = sum(P), OP = sum(OP))
      })

      tibble::tibble(
        meanP  = mean(boot_results$P),
        meanOP = mean(boot_results$OP),
        varP   = stats::var(boot_results$P),
        varOP  = stats::var(boot_results$OP)
      )
    }
  }
}

#' JFK Speech: Inaugural Address (January 20, 1961)
#'
#' Full text of JFK's Inaugural Address on January 20, 1961.
#'
#' @format A character string containing the full text of the speech.
#'
#' @source \url{https://www.govinfo.gov/app/details/PPP-1961-book1}.
#' These original source texts are U.S. government works and are not subject to
#' copyright protection in the United States under 17 U.S.C. § 105.
#' The package authors do not claim copyright in the original presidential speech texts.
"jfk19610120"

#' JFK Speech: Address Before the UN General Assembly (September 25, 1961)
#'
#' Full text of JFK's address before the UN General Assembly on September 25, 1961.
#'
#' @format A character string containing the full text of the speech.
#'
#' @source \url{https://www.govinfo.gov/app/details/PPP-1961-book1}.
#' These original source texts are U.S. government works and are not subject to
#' copyright protection in the United States under 17 U.S.C. § 105.
#' The package authors do not claim copyright in the original presidential speech texts.
"jfk19610925"

#' JFK Speech: Commencement Address at American University (June 10, 1963)
#'
#' Full text of Senator John F. Kennedy's commencement address at American University on June 10, 1963.
#'
#' @format A character string containing the full text of the speech.
#'
#' @source \url{https://www.govinfo.gov/app/details/PPP-1963-book1}.
#' These original source texts are U.S. government works and are not subject to
#' copyright protection in the United States under 17 U.S.C. § 105.
#' The package authors do not claim copyright in the original presidential speech texts.
"jfk19630610"

#' leadeR: Profiling Leaders at a Distance
#'
#' The leadeR package profiles political leaders using text analysis,
#' implementing Leadership Trait Analysis (LTA) and Operational Code
#' Analysis (OCA). You provide text data and the package performs the
#' analyses.
#'
#' @import dplyr
#' @import stringr
#' @import tidyr
#' @importFrom spacyr spacy_parse
#' @importFrom data.table rleid
#' @importFrom tibble tibble
#' @importFrom countries list_countries
#' @importFrom countrycode countryname
#' @importFrom purrr map_dfr
#' @importFrom stringi stri_trans_general
#' @importFrom stats var na.omit
#' @keywords internal
"_PACKAGE"

# Suppress R CMD check NOTEs for dplyr/tidyr NSE column references
utils::globalVariables(c(
  "A", "B", "C", "D", "HC", "IC", "IP", "LC", "N", "Na", "OA", "OC",
  "ON", "OP", "OS", "OSC", "P", "Pp", "S", "SC", "Ss", "TI", "Ta",
  "meanA", "meanHC", "meanIC", "meanIP", "meanLC", "meanN", "meanOA",
  "meanOC", "meanON", "meanOP", "meanOS", "meanOSC", "meanP", "meanS",
  "meanSC", "meanTI",
  "varA", "varHC", "varIC", "varIP", "varLC", "varN", "varOA", "varOC",
  "varON", "varOP", "varOS", "varOSC", "varP", "varS", "varSC", "varTI",
  "actor", "actor_id", "actor_norm", "affiliation", "agent_actor",
  "agent_norm", "agent_token", "amod_ner", "amod_phrase", "by_ner",
  "by_phrase", "conf", "conj_head", "conj_id", "country.name.en",
  "counts", "ctrl", "dep_rel", "doc_id", "ent_flag", "ent_group",
  "ent_phrase", "entity", "entity.x", "entity.y", "entity_clean",
  "entity_group_id", "entity_type", "exact_own_match", "first_token_id",
  "future_aux", "has_against", "has_conditional", "has_modal",
  "has_possessive", "head_ner", "head_noun", "head_token_id", "in_quote",
  "inf_lemma", "inf_tag", "is_conditional", "is_future", "is_gpe",
  "is_let_us", "is_own", "is_pronoun", "is_quote_char", "is_self_ref_noun",
  "label", "lemma", "lemma_full", "lemma_lower", "main_verb_id", "mod",
  "modal_aux", "ner_type_clean", "nom_head", "noun_lemma", "noun_phrase",
  "noun_phrase_tmp", "obj_head", "object_phrase", "opc_score", "particle",
  "phrase", "pos", "poss_dep", "poss_lemma", "poss_ner", "poss_phrase",
  "poss_pronoun", "power", "prep_head", "prep_token", "prev_token",
  "quote_cumsum", "quote_open", "sent_ner", "sent_subject", "sentence",
  "sentence_id", "sentence_text", "starts_with_own_name",
  "starts_with_own_pron", "subject", "subject_clean", "subject_ner_type",
  "subject_no_article", "subject_norm", "subject_phrase", "tag", "tags",
  "token", "token_id", "token_lower", "verb_dep", "verb_head", "verb_id",
  "verb_lemma", "verb_tag", "verb_token", "xcomp_id"
))

#' Clean text for analysis
#'
#' Removes editorial annotations in square brackets, parentheses, and
#' curly braces from political speech transcripts. Also normalizes
#' whitespace and collapses double em-dashes left behind by removals.
#'
#' @param x A character vector of text to clean.
#' @return A character vector with annotations removed and whitespace normalized.
#' @export
#' @examples
#' clean_text("We must act now [applause] for the future.")
#' clean_text("The president (speaking loudly) left the room.")
clean_text <- function(x) {

  if (is.factor(x)) x <- as.character(x)

  repeat {
    x_new <- x
    # Square brackets [ ... ] and fullwidth \uff3b ... \uff3d
    x_new <- gsub("\\[[^\\]]*\\]", "", x_new, perl = TRUE)
    x_new <- gsub("\uff3b[^\uff3d]*\uff3d", "", x_new, perl = TRUE)
    # Parentheses ( ... ) and fullwidth \uff08 ... \uff09
    x_new <- gsub("\\([^)]*\\)", "", x_new, perl = TRUE)
    x_new <- gsub("\uff08[^\uff09]*\uff09", "", x_new, perl = TRUE)
    # Curly braces { ... }
    x_new <- gsub("\\{[^}]*\\}", "", x_new, perl = TRUE)
    if (identical(x_new, x)) break
    x <- x_new
  }

  # Remove stray unmatched brackets
  x <- gsub("[\\[\\]{}]", "", x, perl = TRUE)
  x <- gsub("[()]", "", x, perl = TRUE)
  x <- gsub("[\uff3b\uff3d\uff08\uff09]", "", x, perl = TRUE)

  # Collapse double em-dashes and normalize whitespace
  x <- gsub("\\s*\u2014\\s*\u2014\\s*", "\u2014", x, perl = TRUE)
  x <- gsub("\\s+", " ", x, perl = TRUE)
  trimws(x)
}

test_that("clean_text removes square brackets", {
  expect_equal(clean_text("hello [applause] world"), "hello world")
})

test_that("clean_text removes parentheses", {
  expect_equal(clean_text("hello (laughter) world"), "hello world")
})

test_that("clean_text removes curly braces", {
  expect_equal(clean_text("hello {stage direction} world"), "hello world")
})

test_that("clean_text handles nested brackets", {
  expect_equal(clean_text("a [b [c] d] e"), "a d e")
  expect_equal(clean_text("a [b] [c] e"), "a e")
})

test_that("clean_text removes fullwidth brackets", {
  expect_equal(clean_text("text \uff3bnote\uff3d more"), "text more")
})

test_that("clean_text removes fullwidth parentheses", {
  expect_equal(clean_text("text \uff08note\uff09 more"), "text more")
})

test_that("clean_text converts factor input", {
  expect_equal(clean_text(factor("hello [x] world")), "hello world")
})

test_that("clean_text collapses double em-dashes", {
  expect_equal(clean_text("word \u2014 \u2014 word"), "word\u2014word")
})

test_that("clean_text normalizes whitespace", {
  expect_equal(clean_text("too   many    spaces"), "too many spaces")
})

test_that("clean_text leaves clean text unchanged", {
  expect_equal(clean_text("nothing to clean here"), "nothing to clean here")
})

test_that("clean_text removes stray unmatched brackets", {
  expect_equal(clean_text("stray ] bracket [ here"), "stray bracket here")
})

test_that("clean_text works on character vectors", {
  result <- clean_text(c("first [a] text", "second (b) text"))
  expect_equal(result, c("first text", "second text"))
})

# Operational code

#' Compute VICS operational code scores from verb-entity coding
#'
#' Internal function that takes a data frame of coded utterances (with
#' \code{opc_score} and \code{entity_type} columns) and computes the full
#' set of P and I indices.
#'
#' @param dat A data frame with columns \code{opc_score} (integer, -3 to 3)
#'   and \code{entity_type} (character, "Self" or "Other").
#' @return A one-row \code{\link[tibble]{tibble}} with P1--P5, I1--I5, and
#'   raw counts.
#' @keywords internal
get_scores <- function(dat) {

  iqv <- function(x) {
    if (length(x) < 2) return(0)
    p <- prop.table(table(x))
    if (length(p) < 2) return(0)
    (6 / (6 - 1)) * (1 - sum(p^2))
  }

  n_other <- sum(dat$entity_type == "Other")
  n_self <- sum(dat$entity_type == "Self")

  # P metrics (require Other > 0)
  if (n_other > 0) {
    P1 = (sum(dat$opc_score > 0 & dat$entity_type == "Other") -
            sum(dat$opc_score < 0 & dat$entity_type == "Other")) / n_other
    P2 = sum(dat$opc_score[which(dat$entity_type == "Other")]) / (n_other * 3)
    P3 = 1 - iqv(dat$opc_score[which(dat$entity_type == "Other")])

    other_reward = sum(dat$opc_score == 3 & dat$entity_type == "Other")
    other_promise = sum(dat$opc_score == 2 & dat$entity_type == "Other")
    other_support = sum(dat$opc_score == 1 & dat$entity_type == "Other")
    other_oppose = sum(dat$opc_score == -1 & dat$entity_type == "Other")
    other_threat = sum(dat$opc_score == -2 & dat$entity_type == "Other")
    other_punish = sum(dat$opc_score == -3 & dat$entity_type == "Other")
  } else {
    P1 = NA_real_
    P2 = NA_real_
    P3 = NA_real_
    other_reward = 0
    other_promise = 0
    other_support = 0
    other_oppose = 0
    other_threat = 0
    other_punish = 0
  }

  # I metrics (require Self > 0)
  if (n_self > 0) {
    I1 = (sum(dat$opc_score > 0 & dat$entity_type == "Self") -
            sum(dat$opc_score < 0 & dat$entity_type == "Self")) / n_self
    I2 = sum(dat$opc_score[which(dat$entity_type == "Self")]) / (n_self * 3)
    I3 = 1 - iqv(dat$opc_score[which(dat$entity_type == "Self")])
    I4a = 1 - abs(I1)
    I4b = 1 - abs(
      (sum(dat$opc_score %in% c(-2, -1, 1, 2) & dat$entity_type == "Self") -
         sum(dat$opc_score %in% c(-3, 3) & dat$entity_type == "Self")) / n_self
    )
    I5_punish = sum(dat$opc_score == -3 & dat$entity_type == "Self") / n_self
    I5_threat = sum(dat$opc_score == -2 & dat$entity_type == "Self") / n_self
    I5_oppose = sum(dat$opc_score == -1 & dat$entity_type == "Self") / n_self
    I5_support = sum(dat$opc_score == 1 & dat$entity_type == "Self") / n_self
    I5_promise = sum(dat$opc_score == 2 & dat$entity_type == "Self") / n_self
    I5_reward = sum(dat$opc_score == 3 & dat$entity_type == "Self") / n_self

    self_reward = sum(dat$opc_score == 3 & dat$entity_type == "Self")
    self_promise = sum(dat$opc_score == 2 & dat$entity_type == "Self")
    self_support = sum(dat$opc_score == 1 & dat$entity_type == "Self")
    self_oppose = sum(dat$opc_score == -1 & dat$entity_type == "Self")
    self_threat = sum(dat$opc_score == -2 & dat$entity_type == "Self")
    self_punish = sum(dat$opc_score == -3 & dat$entity_type == "Self")
  } else {
    I1 = NA_real_
    I2 = NA_real_
    I3 = NA_real_
    I4a = NA_real_
    I4b = NA_real_
    I5_punish = NA_real_
    I5_threat = NA_real_
    I5_oppose = NA_real_
    I5_support = NA_real_
    I5_promise = NA_real_
    I5_reward = NA_real_
    self_reward = 0
    self_promise = 0
    self_support = 0
    self_oppose = 0
    self_threat = 0
    self_punish = 0
  }

  # P4 and P5 require both
  P4 = if (nrow(dat) > 0) n_self / nrow(dat) else NA_real_
  P5 = if (!is.na(P3) && !is.na(P4)) 1 - P3 * P4 else NA_real_

  return(tibble::tibble(P1 = P1, P2 = P2, P3 = P3, P4 = P4, P5 = P5,
                I1 = I1, I2 = I2, I3 = I3, I4a = I4a, I4b = I4b,
                I5_punish = I5_punish, I5_threat = I5_threat, I5_oppose = I5_oppose,
                I5_support = I5_support, I5_promise = I5_promise, I5_reward = I5_reward,
                self_reward = self_reward, self_promise = self_promise, self_support = self_support,
                self_oppose = self_oppose, self_threat = self_threat, self_punish = self_punish,
                other_reward = other_reward, other_promise = other_promise, other_support = other_support,
                other_oppose = other_oppose, other_threat = other_threat, other_punish = other_punish
                ))

}

# --- MASSIVELY EXTENDED VICS VERB DICTIONARIES ---

# --- 1. PUNISH (-3) ---
verbs_punish <- c(
  "attack", "bomb", "destroy", "invade", "punish", "expel", "assault", "strike",
  "bombard", "shell", "raid", "ambush", "fire", "shoot", "massacre", "slaughter",
  "decimate", "eradicate", "annihilate", "obliterate", "devastate", "ravage",
  "pulverize", "flatten", "crush", "smash", "wreck", "liquidate", "neutralize",
  "conquer", "subjugate", "vanquish", "overrun", "besiege", "encircle",
  "mobilize", "deploy", "disarm", "demilitarize", "occupy", "annex", "seize",
  "capture", "arrest", "detain", "jail", "imprison", "incarcerate", "abduct",
  "kidnap", "torture", "execute", "assassinate", "murder", "kill", "purge",
  "eliminate", "exterminate", "genocide", "sanction", "embargo", "blockade",
  "boycott", "ban", "bar", "blacklist", "censor", "suppress", "repress", "quell",
  "silence", "gag", "confiscate", "expropriate", "sequestrate", "freeze", "appropriate",
  "nationalize", "revoke", "rescind", "cancel", "terminate", "suspend", "sever",
  "cut", "break", "rupture", "halt", "stop", "impede", "obstruct", "disrupt",
  "sabotage", "subvert", "undermine", "destabilize", "topple", "overthrow",
  "oust", "depose", "remove", "dismiss", "sack", "deport", "extradite", "exile",
  "banish", "marginalize", "isolate", "alienate", "coerce", "force", "compel",
  "constrain", "impose", "dictate", "mandate", "enforce", "prosecute", "indict",
  "convict", "sentence", "penalize", "fine", "levy", "tax", "burden", "encumber",
  "bankrupt", "impoverish", "starve", "strangle", "choke", "suffocate",

  "blow up", "wipe out", "take over", "crack down", "lock up", "shut down",
  "cut off", "kick out", "push out", "drive out", "beat back", "hunt down",
  "round up", "shoot down", "take out", "clamp down", "close down",

  "hack", "infiltrate", "breach", "compromise", "surveil", "spy", "wiretap",
  "dox", "jam", "scramble", "intercept", "nuke", "gas", "drone",

  # NEW: Political/legislative obstruction verbs (human coders count these as punish)
  "defeat", "kill", "gut", "slash", "strip", "deny", "deprive",
  "defund", "decimate", "dismantle", "abolish", "eliminate",
  "discriminate", "persecute", "oppress", "exploit",
  "violate", "breach", "infringe", "transgress",
  "betray", "abandon", "forsake", "desert"
)

# --- 2. THREATEN (-2) ---
# REFINED: Only warnings/threats of future negative action, not demands/pressure
verbs_threaten <- c(
  # Core threat/warning verbs
  "threaten", "warn", "alert", "intimidate", "menace", "blackmail",
  "bully", "cow", "browbeat",

  # Fear-inducing verbs
  "scare", "frighten", "terrify", "alarm", "horrify", "appall",
  "daunt", "unnerve",

  # Escalation verbs
  "provoke", "incite", "instigate", "foment", "inflame", "escalate",

  # Danger/risk verbs
  "imperil", "jeopardize", "endanger",

  # Warning/omen verbs
  "forewarn", "admonish", "caution",

  # Display of force
  "posture", "bluster", "brandish", "flaunt", "flex", "bluff",

  # Ultimatum (true threat)
  "ultimatum"
)

# --- 3. OPPOSE (-1) ---
verbs_oppose <- c(
  "oppose", "condemn", "reject", "refuse", "criticize", "deny", "object",
  "protest", "blame", "accuse", "charge", "allege", "impute", "attribute",
  "ascribe", "indict", "impeach", "arraign", "denounce", "decry", "deplore",
  "lament", "mourn", "grieve", "regret", "rue", "disapprove", "disagree",
  "dissent", "differ", "clash", "conflict", "collide", "contest", "dispute",
  "question", "query", "doubt", "suspect", "distrust", "mistrust", "skeptical",
  "cynical", "dismiss", "discard", "ignore", "disregard", "neglect", "overlook",
  "shun", "snub", "spurn", "rebuff", "repulse", "alienate", "estrange",
  "divorce", "separate", "split", "divide", "fragment", "splinter", "polarize",
  "radicalize", "marginalize", "exclude", "omit", "bar", "block", "hinder",
  "hamper", "impede", "thwart", "stymie", "foil", "frustrate", "check", "curb",
  "restrain", "restrict", "limit", "confine", "constrain", "contain",
  "encircle", "surround", "envelop", "besieged", "withhold", "retain", "keep",
  "reserve", "preserve", "maintain", "uphold", "defend", "protect", "guard",
  "shield", "screen", "cover", "hide", "conceal", "mask", "veil", "cloak",
  "disguise", "camouflage", "obscure", "cloud", "blur", "confuse", "muddle",
  "complicate", "obfuscate", "distort", "twist", "warp", "pervert", "manipulate",
  "exploit", "abuse", "misuse", "mistreat", "maltreat", "neglect",

  "turn down", "vote down", "rule out", "push back", "break off", "call off",
  "walk out", "opt out", "back out", "stand against", "shut out", "fend off",

  "veto", "filibuster", "abstain", "defect", "stonewall", "stall", "table",

  # Assertion/criticism verbs (added to improve Other detection)
  "assert", "declare", "proclaim", "claim", "contend", "maintain",
  "point out", "highlight", "emphasize", "stress", "underscore",
  "argue", "debate", "dispute", "counter", "rebut", "refute",
  "call out", "take issue", "take exception",

  # Demand/pressure verbs (moved from Threaten - these are opposition, not threats)
  "demand", "insist", "require", "pressure", "push",
  "order", "command", "direct", "instruct", "leverage"
)

# --- 4. SUPPORT (+1) ---
verbs_support <- c(
  "support", "endorse", "approve", "agree", "praise", "accept", "welcome",
  "aid", "help", "assist", "benefit", "favor", "prefer", "choose", "select",
  "elect", "vote", "nominate", "appoint", "designate", "name", "authorize",
  "empower", "entitle", "enable", "facilitate", "promote", "advance",
  "further", "foster", "cultivate", "nurture", "nurse", "cherish", "treasure",
  "value", "prize", "esteem", "respect", "admire", "honor", "revere",
  "venerate", "worship", "idolize", "adore", "love", "like", "enjoy",
  "appreciate", "grateful", "thank", "acknowledge", "recognize", "admit",
  "concede", "confess", "grant", "allow", "permit", "let", "suffer",
  "tolerate", "bear", "endure", "stand", "abide", "condone", "excuse",
  "pardon", "forgive", "absolve", "acquit", "exonerate", "vindicate", "justify",
  "warrant", "defend", "uphold", "sustain", "maintain", "preserve", "keep",
  "save", "rescue", "liberate", "free", "emancipate", "release", "discharge",
  "dismiss", "exempt", "spare", "relieve", "ease", "alleviate", "mitigate",
  "ameliorate", "improve", "better", "enhance", "enrich", "upgrade", "update",
  "modernize", "reform", "renovate", "restore", "revive", "resurrect",
  "renew", "regenerate", "rejuvenate", "invigorate", "refresh", "revitalize",
  "stimulate", "encourage", "inspire", "motivate", "embolden", "hearten",
  "comfort", "console", "soothe", "calm", "quiet", "pacify", "placate",
  "appease", "mollify", "conciliate", "reconcile", "mediate", "arbitrate",
  "negotiate", "bargain", "haggle", "deal", "trade", "exchange", "swap",
  "switch", "substitute", "replace", "alternate", "rotate", "cycle",
  "circulate", "distribute", "dispense", "allocate", "allot", "assign",
  "apportion", "share", "partake", "participate", "cooperate", "collaborate",
  "coordinate", "associate", "ally", "unite", "join", "combine", "merge",
  "fuse", "blend", "mix", "mingle", "integrate", "incorporate", "consolidate",
  "unify", "solidify", "strengthen", "fortify", "reinforce", "buttress",
  "bolster", "prop", "shore", "brace",

  "back up", "stand by", "stick by", "root for", "cheer on", "vote for",
  "side with", "team up", "band together", "hear out",

  "partner", "align", "caucus",

  # Appeal/request verbs (added to improve Other detection)
  "call", "call for", "call upon", "call on",
  "appeal", "appeal to", "appeal for",
  "urge", "exhort", "implore", "beseech", "entreat",
  "request", "ask", "seek", "petition", "solicit",
  "invite", "summon", "beckon", "convene",
  "plead", "beg", "pray",

  # Additional appeal/exhortation verbs (critical for political speech)
  "challenge", "congratulate", "salute", "honor", "celebrate",
  "recommend", "suggest", "propose", "advise",
  "let", "allow",  # for "let us" / "let's" constructions
  "need",  # prescriptive "we need to..."
  "wish", "hope"
)

# --- 5. PROMISE (+2) ---
# Cooperative Words: Future commitments, treaty signing, and assurances.
verbs_promise <- c(
  # Core commitment verbs (VICS Promise = binding commitments)
  "promise", "pledge", "assure", "guarantee", "warrant", "certify", "attest",
  "swear", "vow", "commit", "bind", "obligate", "undertake", "covenant",

  # Agreement/consent verbs (formal acceptance)
  "agree", "consent", "assent", "accede", "concur", "comply",

  # Formal offers and proposals (binding)
  "propose", "offer", "tender", "bid",

  # Financial commitments
  "finance", "fund", "subsidize", "underwrite",

  # Advocacy verbs (committed support)
  "advocate", "champion", "sponsor",

  # Phrasal commitment verbs
  "enter into", "sign on", "swear in",

  # Treaty/formal agreement verbs
  "ratify", "sign", "ink", "finalise", "formalize",

  # RESTORED: Intent verbs that indicate commitment (human coders count these)
  # Context: When spoken by political leader, these ARE commitments
  "intend", "plan", "aim",
  "expect", "anticipate",
  "declare", "announce", "proclaim",
  "affirm", "reaffirm", "confirm"

  # REMOVED to fix overcounting:
  # - suggest, recommend, advise, urge, counsel (generic discussion, not commitment)
  # - hope, wish, desire, want, need, expect (desire, not commitment)
  # - plan, intend, aim (intent but not binding commitment)
  # - ask, request, demand, beg, plead (requests, not promises)
  # - call, invite, summon (convening, not promising)
  # - anticipate, predict, forecast (prediction, not commitment)
)

# --- 6. REWARD (+3) ---
# Cooperative Deeds: Material aid, de-escalation actions, and granting rights.
verbs_reward <- c(
  # Core reward verbs (actual positive deeds)
  "reward", "award", "grant", "give", "donate", "contribute",
  "provide", "supply", "furnish", "deliver",

  # Material aid verbs (completed actions)
  "aid", "assist", "rescue", "liberate", "free", "save",

  # Concession/de-escalation verbs
  "release", "withdraw", "relinquish", "cede", "surrender",
  "lift", "waive", "ease", "relieve",

  # Financial/material transfer verbs
  "fund", "finance", "invest", "loan", "lend",
  "compensate", "reimburse", "repay", "refund",

  # RESTORED: High-frequency material action verbs (to fix Other Reward undercounting)
  "build", "construct", "create", "establish", "found",
  "open", "launch", "initiate", "start",
  "restore", "rebuild", "repair", "fix",
  "send", "ship", "transport", "dispatch",
  "allocate", "distribute", "share", "transfer",

  # Treaty/agreement completion verbs
  "sign", "ratify", "conclude", "finalize", "seal",

  # Phrasal reward verbs
  "give up", "hand over", "turn over", "pull out", "stand down",
  "pay back", "give back", "bail out",

  # De-escalation
  "repatriate", "pardon", "amnesty",

  # NEW: Achievement/accomplishment verbs (human coders count past accomplishments as reward)
  "pass", "enact", "adopt", "implement", "execute",
  "achieve", "accomplish", "attain", "realize", "fulfill",
  "secure", "win", "gain", "obtain", "acquire",
  "complete", "finish", "succeed", "triumph", "prevail",
  "improve", "strengthen", "enhance", "boost", "increase",
  "expand", "extend", "broaden", "widen", "deepen",
  "develop", "advance", "progress", "promote", "elevate",
  "protect", "defend", "safeguard", "preserve", "maintain",
  "uphold", "sustain", "ensure", "guarantee"
)

# --- 7. VERBS THAT BECOME NEGATIVE WITH "AGAINST" (NEW) ---
verbs_flip_with_against <- c(
  # Voting/political action verbs -> Oppose (-1)
  "vote", "speak", "argue", "testify", "campaign", "rally",
  "lobby", "advocate", "push", "work", "act", "move",
  "decide", "rule", "stand", "come out", "go",
  # Combat/struggle verbs -> Punish (-3)
  "fight", "battle", "struggle", "war", "rebel", "revolt",
  "strike", "march", "protest", "demonstrate", "riot"
)

# Subset that becomes Punish (-3) when + against (vs Oppose -1)
verbs_punish_with_against <- c(
  "fight", "battle", "struggle", "war", "rebel", "revolt",
  "strike", "march", "riot"
)

# --- NOMINALIZATION DICTIONARIES ---
nouns_reward <- c(
  "aid", "assistance", "support", "help", "relief", "provision", "provisions",
  "liberation", "freedom", "grant", "grants", "gift", "gifts", "donation", "donations",
  "contribution", "contributions", "cooperation", "collaboration", "partnership",
  "alliance", "alliances", "treaty", "treaties", "agreement", "agreements",
  "rescue", "deployment", "delivery", "investment", "investments",

  # NEW: Policy/legislative achievement nouns
  "achievement", "achievements", "accomplishment", "accomplishments",
  "success", "successes", "victory", "victories", "triumph", "triumphs",
  "progress", "improvement", "improvements", "advancement", "advancements",
  "reform", "reforms", "legislation", "law", "laws", "act", "acts",
  "program", "programs", "initiative", "initiatives", "policy", "policies",
  "benefit", "benefits", "protection", "protections", "security",
  "development", "developments", "growth", "expansion", "increase"
)

nouns_punish <- c(
  "attack", "attacks", "assault", "assaults", "invasion", "invasions",
  "destruction", "devastation", "sanction", "sanctions", "embargo", "embargoes",
  "blockade", "blockades", "punishment", "punishments", "retaliation",
  "strike", "strikes", "bombing", "bombings", "occupation", "seizure",
  "expulsion", "execution", "executions", "massacre", "massacres",

  # Punitive administrative actions
  "withdrawal", "withdrawals",
  "suspension", "suspensions",
  "termination", "terminations",
  "revocation", "revocations",
  "cutoff", "cutoffs",
  "cancellation", "cancellations",
  "abrogation", "abrogations",
  "severance",
  "crackdown", "crackdowns",
  "clampdown", "clampdowns",

  # NEW: Political/policy harm nouns
  "discrimination", "persecution", "oppression", "exploitation",
  "violation", "violations", "breach", "breaches", "infringement",
  "defeat", "defeats", "rejection", "rejections", "denial", "denials",
  "obstruction", "obstructions", "veto", "vetoes",
  "cut", "cuts", "slash", "slashes", "reduction", "reductions",
  "prosecution", "prosecutions", "indictment", "indictments",
  "conviction", "convictions", "sentence", "sentences"
)

nouns_support <- c(
  "endorsement", "endorsements", "approval", "recognition", "acknowledgment",
  "praise", "commendation", "appreciation", "gratitude", "thanks",
  "appeal", "appeals", "request", "requests", "call", "calls",
  "petition", "petitions", "recommendation", "recommendations",
  "tribute", "tributes", "honor", "honors", "salute", "salutes"
)

nouns_threaten <- c(
  "threat", "threats", "warning", "warnings", "ultimatum", "ultimatums",
  "intimidation", "coercion", "blackmail", "pressure", "pressures",

  # Military/conflict threat nouns
  "aggression", "aggressions",
  "escalation", "escalations",
  "hostility", "hostilities",
  "confrontation", "confrontations",
  "provocation", "provocations",
  "menace", "menaces",

  # Implicit threat nouns
  "buildup", "buildups",
  "mobilization", "mobilizations",
  "brinkmanship",

  # Diplomatic threat nouns
  "deadline", "deadlines",
  "consequence", "consequences"
)

nouns_oppose <- c(
  "opposition", "objection", "objections", "protest", "protests",
  "criticism", "criticisms", "condemnation", "rejection", "denial",
  "veto", "vetoes", "refusal", "dispute", "disputes"
)

# NEW: Promise nominalizations (previously missing)
nouns_promise <- c(

  "promise", "promises", "pledge", "pledges",
  "commitment", "commitments", "assurance", "assurances",
  "guarantee", "guarantees", "vow", "vows",
  "oath", "oaths", "covenant", "covenants",
  "undertaking", "undertakings", "obligation", "obligations",
  "agreement", "agreements", "treaty", "treaties",
  "pact", "pacts", "accord", "accords",
  "declaration", "declarations",
  "intention", "intentions",
  "plan", "plans", "proposal", "proposals"
)

#' Compute operational code (VICS) scores
#'
#' Performs full VICS (Verbs in Context System) operational code analysis on
#' a speech text, classifying verb-entity pairs as Self/Other and scoring
#' them on a -3 to +3 scale (Punish to Reward). Returns P1--P5 and I1--I5
#' indices.
#'
#' @param own_entity A character vector of entity names representing the
#'   speaker's own country or group.
#' @param text A character string containing the speech text to analyse.
#' @param bootstrap Logical; if \code{TRUE}, return bootstrapped mean and
#'   variance estimates for all indices. Default is \code{FALSE}.
#' @param B Integer; number of bootstrap replicates. Default is 1000.
#' @return A one-row \code{\link[tibble]{tibble}} with P1--P5, I1--I5, and
#'   raw Self/Other counts by score category.
#' @export
get_oca <- function(own_entity, text, bootstrap = FALSE, B = 1000) {

  # Build entities corpus
  all_entities_corpus <- build_entities_corpus()

  # --- A. Setup Entity Lists ---
  own_entity_clean <- unique(c(tolower(own_entity), expand_aliases_country(own_entity)))

  own_terms <- c(own_entity_clean,
                 "we", "our nation", "our country",
                 "our state", "our government", "our institution",
                 "our group", "our party", "our people",
                 "i", "my nation", "my country",
                 "my state", "my government", "my institution",
                 "my group", "my party", "my people")

  other_terms <- c(setdiff(all_entities_corpus, tolower(own_terms)),
                   "they", "their nation", "their country",
                   "their state", "their government", "their institution")

  # --- B. Parse Text ---
  parsed <- spacyr::spacy_parse(text, dependency = TRUE, entity = TRUE, tag = TRUE)

  # --- C. Identify Multi-word Entities ---
  parsed_entity_grouped <- parsed |>
    dplyr::mutate(ent_group = ifelse(!is.na(entity) & entity != "", 1, NA)) |>
    dplyr::group_by(doc_id, sentence_id) |>
    dplyr::mutate(entity_group_id = data.table::rleid(ent_group)) |>
    dplyr::ungroup() |>
    dplyr::mutate(entity_group_id = ifelse(is.na(ent_group), NA, entity_group_id))

  multiword_entities <- parsed_entity_grouped |>
    dplyr::filter(!is.na(entity_group_id)) |>
    dplyr::group_by(doc_id, sentence_id, entity_group_id) |>
    dplyr::summarise(entity = paste(token, collapse = " "), .groups = "drop")

  # --- D. Link Subjects to Verbs ---

  # 1. Isolate Subjects (with NER entity type info)
  subjects <- parsed_entity_grouped |>
    dplyr::filter(dep_rel %in% c("poss", "nsubj", "nsubjpass")) |>
    dplyr::mutate(
      subject_ner_type = entity  # Keep the NER entity type
    ) |>
    dplyr::select(-entity) |>
    dplyr::left_join(multiword_entities, by = c("doc_id", "sentence_id", "entity_group_id")) |>
    dplyr::mutate(
      poss_pronoun = dplyr::lag(lemma),
      poss_dep = dplyr::lag(dep_rel),
      has_possessive = poss_pronoun %in% c("my", "our", "their") & poss_dep == "poss",
      subject_phrase = dplyr::case_when(
        has_possessive ~ paste(poss_pronoun, lemma),
        !is.na(entity) ~ entity,
        TRUE ~ token
      )
    ) |>
    dplyr::filter(dep_rel %in% c("nsubj", "nsubjpass")) |>
    dplyr::select(doc_id, sentence_id, head_token_id, subject_phrase, subject_ner_type) |>
    dplyr::distinct()

  # 1b. ENHANCED passive voice agent detection
  by_prep_tokens <- parsed |>
    dplyr::filter(dep_rel == "prep" & tolower(lemma) == "by") |>
    dplyr::select(doc_id, sentence_id, by_token_id = token_id, verb_head = head_token_id)

  by_prep_objects <- parsed |>
    dplyr::filter(dep_rel == "pobj") |>
    dplyr::inner_join(by_prep_tokens,
               by = c("doc_id", "sentence_id", "head_token_id" = "by_token_id")) |>
    dplyr::select(doc_id, sentence_id, head_token_id = verb_head, agent_token_id = token_id, agent_token = token)

  by_agents_with_entities <- by_prep_objects |>
    dplyr::left_join(
      parsed_entity_grouped |> dplyr::select(doc_id, sentence_id, token_id, entity_group_id, entity),
      by = c("doc_id", "sentence_id", "agent_token_id" = "token_id")
    ) |>
    dplyr::left_join(multiword_entities, by = c("doc_id", "sentence_id", "entity_group_id")) |>
    dplyr::mutate(
      subject_phrase = dplyr::coalesce(entity.y, agent_token),
      subject_ner_type = dplyr::coalesce(entity.x, "")
    ) |>
    dplyr::select(doc_id, sentence_id, head_token_id, subject_phrase, subject_ner_type) |>
    dplyr::distinct()

  direct_agents <- parsed_entity_grouped |>
    dplyr::filter(dep_rel %in% c("agent", "obl")) |>
    dplyr::mutate(subject_ner_type = entity) |>
    dplyr::select(-entity) |>
    dplyr::left_join(multiword_entities, by = c("doc_id", "sentence_id", "entity_group_id")) |>
    dplyr::mutate(
      subject_phrase = dplyr::case_when(
        !is.na(entity) ~ entity,
        TRUE ~ token
      )
    ) |>
    dplyr::select(doc_id, sentence_id, head_token_id, subject_phrase, subject_ner_type) |>
    dplyr::distinct()

  agents <- dplyr::bind_rows(direct_agents, by_agents_with_entities) |> dplyr::distinct()

  # Combine standard subjects with agent phrases
  subjects <- dplyr::bind_rows(subjects, agents) |> dplyr::distinct()

  # 1c. RELATIVE CLAUSE HANDLING (NEW)
  relcl_verbs <- parsed |>
    dplyr::filter(dep_rel == "relcl" & pos == "VERB") |>
    dplyr::select(doc_id, sentence_id, token_id, relcl_head = head_token_id,
           verb_lemma = lemma, verb_tag = tag)

  if (nrow(relcl_verbs) > 0) {
    relcl_head_nouns <- parsed |>
      dplyr::filter(pos %in% c("NOUN", "PROPN", "PRON")) |>
      dplyr::select(doc_id, sentence_id, token_id, head_noun = token, head_ner = entity)

    relcl_subjects <- relcl_verbs |>
      dplyr::inner_join(relcl_head_nouns,
                 by = c("doc_id", "sentence_id", "relcl_head" = "token_id")) |>
      dplyr::mutate(
        subject_phrase = head_noun,
        subject_ner_type = dplyr::coalesce(head_ner, "")
      ) |>
      dplyr::select(doc_id, sentence_id, head_token_id = token_id, subject_phrase, subject_ner_type) |>
      dplyr::distinct()

    subjects <- dplyr::bind_rows(subjects, relcl_subjects) |> dplyr::distinct()
  }

  # 1d. ADJECTIVAL CLAUSE HANDLING (NEW)
  acl_verbs <- parsed |>
    dplyr::filter(dep_rel == "acl" & pos == "VERB") |>
    dplyr::select(doc_id, sentence_id, token_id, acl_head = head_token_id,
           verb_lemma = lemma, verb_tag = tag)

  if (nrow(acl_verbs) > 0) {
    acl_head_nouns <- parsed |>
      dplyr::filter(pos %in% c("NOUN", "PROPN", "PRON")) |>
      dplyr::select(doc_id, sentence_id, token_id, head_noun = token, head_ner = entity)

    acl_subjects <- acl_verbs |>
      dplyr::inner_join(acl_head_nouns,
                 by = c("doc_id", "sentence_id", "acl_head" = "token_id")) |>
      dplyr::mutate(
        subject_phrase = head_noun,
        subject_ner_type = dplyr::coalesce(head_ner, "")
      ) |>
      dplyr::select(doc_id, sentence_id, head_token_id = token_id, subject_phrase, subject_ner_type) |>
      dplyr::distinct()

    subjects <- dplyr::bind_rows(subjects, acl_subjects) |> dplyr::distinct()
  }

  # 2. Identify Particles
  particles <- parsed |>
    dplyr::filter(dep_rel == "prt") |>
    dplyr::select(doc_id, sentence_id, head_token_id, particle = lemma)

  future_markers <- parsed |>
    dplyr::filter(dep_rel == "aux", lemma %in% c("will", "shall", "wo", "'ll", "gon")) |>
    dplyr::select(doc_id, sentence_id, head_token_id, future_aux = lemma)

  # 2b. Detect modal verbs as implicit appeals/commitments
  modal_markers <- parsed |>
    dplyr::filter(dep_rel == "aux" & lemma %in% c("should", "must", "ought", "need")) |>
    dplyr::select(doc_id, sentence_id, head_token_id, modal_aux = lemma)

  # 2c. CONDITIONAL THREAT DETECTION (NEW)
  conditional_markers <- parsed |>
    dplyr::filter(dep_rel == "mark" & tolower(lemma) %in% c("if", "unless", "should", "whether")) |>
    dplyr::select(doc_id, sentence_id, mark_head = head_token_id, condition_type = lemma)

  advcl_verbs <- parsed |>
    dplyr::filter(dep_rel == "advcl") |>
    dplyr::select(doc_id, sentence_id, advcl_token = token_id, main_verb_id = head_token_id)

  conditional_consequents <- advcl_verbs |>
    dplyr::inner_join(conditional_markers,
               by = c("doc_id", "sentence_id", "advcl_token" = "mark_head")) |>
    dplyr::select(doc_id, sentence_id, head_token_id = main_verb_id) |>
    dplyr::mutate(has_conditional = TRUE) |>
    dplyr::distinct()

  # 2d. "AGAINST" MODIFIER DETECTION (NEW)
  against_prep <- parsed |>
    dplyr::filter(dep_rel == "prep" & tolower(lemma) == "against") |>
    dplyr::select(doc_id, sentence_id, against_head = head_token_id) |>
    dplyr::mutate(has_against = TRUE) |>
    dplyr::distinct()

  against_agent <- parsed |>
    dplyr::filter(dep_rel == "agent" & tolower(token) == "against") |>
    dplyr::select(doc_id, sentence_id, against_head = head_token_id) |>
    dplyr::mutate(has_against = TRUE) |>
    dplyr::distinct()

  against_markers <- dplyr::bind_rows(against_prep, against_agent) |> dplyr::distinct()

  # 3. Isolate Verbs and Merge Particles
  verbs <- parsed |>
    dplyr::filter(pos == "VERB") |>
    dplyr::left_join(particles, by = c("doc_id", "sentence_id", "token_id" = "head_token_id")) |>
    dplyr::left_join(future_markers, by = c("doc_id", "sentence_id", "token_id" = "head_token_id")) |>
    dplyr::left_join(modal_markers, by = c("doc_id", "sentence_id", "token_id" = "head_token_id")) |>
    dplyr::left_join(conditional_consequents, by = c("doc_id", "sentence_id", "token_id" = "head_token_id")) |>
    dplyr::left_join(against_markers, by = c("doc_id", "sentence_id", "token_id" = "against_head")) |>
    dplyr::mutate(
      lemma_full = ifelse(!is.na(particle), paste(lemma, particle), lemma),
      is_future = !is.na(future_aux),
      has_modal = !is.na(modal_aux),
      is_conditional = dplyr::coalesce(has_conditional, FALSE),
      has_against = dplyr::coalesce(has_against, FALSE)
    ) |>
    dplyr::select(doc_id, sentence_id, token_id, lemma_full, is_future, has_modal, is_conditional, has_against, tag)

  # 4. Join Subjects to Verbs (including NER type)
  utterances <- verbs |>
    dplyr::inner_join(subjects, by = c("doc_id", "sentence_id", "token_id" = "head_token_id")) |>
    dplyr::mutate(subject_ner_type = dplyr::coalesce(subject_ner_type, ""))

  # 4a. COORDINATION HANDLING
  conj_verbs <- parsed |>
    dplyr::filter(dep_rel == "conj" & pos == "VERB") |>
    dplyr::select(doc_id, sentence_id, token_id, conj_head = head_token_id)

  if (nrow(conj_verbs) > 0) {
    head_verb_subjects <- utterances |>
      dplyr::select(doc_id, sentence_id, token_id, subject_phrase, subject_ner_type)

    utterances_from_conj <- verbs |>
      dplyr::inner_join(conj_verbs, by = c("doc_id", "sentence_id", "token_id")) |>
      dplyr::inner_join(head_verb_subjects,
                 by = c("doc_id", "sentence_id", "conj_head" = "token_id")) |>
      dplyr::select(-conj_head)

    utterances <- dplyr::bind_rows(utterances, utterances_from_conj) |> dplyr::distinct()
  }

  # 4a2. IMPERATIVE DETECTION
  verbs_with_subjects <- utterances |>
    dplyr::select(doc_id, sentence_id, token_id) |>
    dplyr::distinct()

  let_us_verbs <- parsed |>
    dplyr::filter(dep_rel == "ccomp" | dep_rel == "xcomp") |>
    dplyr::left_join(
      parsed |>
        dplyr::filter(tolower(lemma) == "let" & pos == "VERB") |>
        dplyr::select(doc_id, sentence_id, let_token = token_id),
      by = c("doc_id", "sentence_id", "head_token_id" = "let_token")
    ) |>
    dplyr::filter(!is.na(head_token_id)) |>
    dplyr::left_join(
      parsed |>
        dplyr::filter(dep_rel %in% c("dobj", "nsubj") & tolower(lemma) %in% c("us", "'s", "we")) |>
        dplyr::select(doc_id, sentence_id, obj_head = head_token_id) |>
        dplyr::mutate(is_let_us = TRUE),
      by = c("doc_id", "sentence_id", "head_token_id" = "obj_head")
    ) |>
    dplyr::filter(dplyr::coalesce(is_let_us, FALSE)) |>
    dplyr::select(doc_id, sentence_id, token_id) |>
    dplyr::distinct()

  imperatives <- verbs |>
    dplyr::filter(tag == "VB") |>
    dplyr::anti_join(verbs_with_subjects, by = c("doc_id", "sentence_id", "token_id")) |>
    dplyr::left_join(let_us_verbs |> dplyr::mutate(is_let_us = TRUE),
              by = c("doc_id", "sentence_id", "token_id")) |>
    dplyr::mutate(
      subject_phrase = ifelse(dplyr::coalesce(is_let_us, FALSE), "we", "you"),
      subject_ner_type = ""
    ) |>
    dplyr::select(-is_let_us)

  if (nrow(imperatives) > 0) {
    utterances <- dplyr::bind_rows(utterances, imperatives) |> dplyr::distinct()
  }

  # 4b. OBJECT PRONOUNS AS IMPLIED "OTHER" AGENTS
  object_pronouns <- parsed |>
    dplyr::filter(dep_rel %in% c("dobj", "iobj") &
           tolower(lemma) %in% c("they", "them", "he", "she", "it", "him", "her", "you", "your")) |>
    dplyr::select(doc_id, sentence_id, head_token_id, object_phrase = token) |>
    dplyr::mutate(subject_phrase = object_phrase, subject_ner_type = "") |>
    dplyr::distinct()

  if (nrow(object_pronouns) > 0) {
    utterances_from_objects <- verbs |>
      dplyr::inner_join(object_pronouns,
                 by = c("doc_id", "sentence_id", "token_id" = "head_token_id"))

    utterances <- dplyr::bind_rows(utterances, utterances_from_objects) |> dplyr::distinct()
  }

  # 4c. NOMINALIZATION DETECTION WITH PROPER AGENT EXTRACTION (IMPROVED)
  all_action_nouns <- c(nouns_reward, nouns_punish, nouns_support, nouns_threaten, nouns_oppose, nouns_promise)

  nominalizations <- parsed |>
    dplyr::filter(pos == "NOUN" & tolower(lemma) %in% all_action_nouns) |>
    dplyr::select(doc_id, sentence_id, token_id, noun_lemma = lemma)

  if (nrow(nominalizations) > 0) {

    nom_poss_agents <- parsed |>
      dplyr::filter(dep_rel %in% c("poss", "nmod:poss")) |>
      dplyr::select(doc_id, sentence_id, poss_head = head_token_id, poss_phrase = token, poss_ner = entity)

    nom_amod_agents <- parsed |>
      dplyr::filter(dep_rel %in% c("amod", "compound")) |>
      dplyr::select(doc_id, sentence_id, amod_head = head_token_id, amod_phrase = token, amod_ner = entity)

    nom_prep <- parsed |>
      dplyr::filter(dep_rel == "prep" & tolower(lemma) == "by") |>
      dplyr::select(doc_id, sentence_id, by_token_id = token_id, nom_head = head_token_id)

    nom_by_objects <- parsed |>
      dplyr::filter(dep_rel == "pobj") |>
      dplyr::inner_join(nom_prep, by = c("doc_id", "sentence_id", "head_token_id" = "by_token_id")) |>
      dplyr::select(doc_id, sentence_id, by_head = nom_head, by_phrase = token, by_ner = entity)

    sentence_subjects <- subjects |>
      dplyr::group_by(doc_id, sentence_id) |>
      dplyr::slice(1) |>
      dplyr::ungroup() |>
      dplyr::select(doc_id, sentence_id, sent_subject = subject_phrase, sent_ner = subject_ner_type)

    utterances_from_nouns <- nominalizations |>
      dplyr::left_join(nom_poss_agents, by = c("doc_id", "sentence_id", "token_id" = "poss_head")) |>
      dplyr::left_join(nom_by_objects, by = c("doc_id", "sentence_id", "token_id" = "by_head")) |>
      dplyr::left_join(nom_amod_agents, by = c("doc_id", "sentence_id", "token_id" = "amod_head")) |>
      dplyr::left_join(sentence_subjects, by = c("doc_id", "sentence_id")) |>
      dplyr::mutate(
        subject_phrase = dplyr::coalesce(poss_phrase, by_phrase, amod_phrase, sent_subject),
        subject_ner_type = dplyr::coalesce(poss_ner, by_ner, amod_ner, sent_ner, ""),
        lemma_full = tolower(noun_lemma),
        is_future = FALSE,
        has_modal = FALSE,
        tag = "NN"
      ) |>
      dplyr::filter(!is.na(subject_phrase)) |>
      dplyr::select(doc_id, sentence_id, token_id, lemma_full, is_future, has_modal, tag,
             subject_phrase, subject_ner_type)

    utterances <- dplyr::bind_rows(utterances, utterances_from_nouns) |> dplyr::distinct()
  }

  # 4d. INFINITIVE CLAUSE EXTRACTION (xcomp)
  xcomp_verbs <- parsed |>
    dplyr::filter(dep_rel == "xcomp" & pos == "VERB") |>
    dplyr::select(doc_id, sentence_id, token_id, xcomp_head = head_token_id,
           inf_lemma = lemma, inf_tag = tag)

  if (nrow(xcomp_verbs) > 0) {
    main_verb_objects <- parsed |>
      dplyr::filter(dep_rel %in% c("dobj", "iobj", "pobj")) |>
      dplyr::select(doc_id, sentence_id, obj_head = head_token_id,
             object_phrase = token, obj_ner = entity)

    pobj_after_upon <- parsed |>
      dplyr::filter(dep_rel == "pobj") |>
      dplyr::left_join(
        parsed |> dplyr::select(doc_id, sentence_id, token_id, prep_token = token),
        by = c("doc_id", "sentence_id", "head_token_id" = "token_id")
      ) |>
      dplyr::filter(tolower(prep_token) %in% c("upon", "on", "to", "for")) |>
      dplyr::left_join(
        parsed |> dplyr::select(doc_id, sentence_id, token_id, prep_head = head_token_id),
        by = c("doc_id", "sentence_id", "head_token_id" = "token_id")
      ) |>
      dplyr::select(doc_id, sentence_id, obj_head = prep_head,
             object_phrase = token, obj_ner = entity) |>
      dplyr::filter(!is.na(obj_head))

    all_objects <- dplyr::bind_rows(main_verb_objects, pobj_after_upon) |> dplyr::distinct()

    main_verb_subjects <- subjects |>
      dplyr::select(doc_id, sentence_id, head_token_id, main_subject = subject_phrase,
             main_ner = subject_ner_type)

    utterances_from_xcomp <- xcomp_verbs |>
      dplyr::left_join(main_verb_subjects,
                by = c("doc_id", "sentence_id", "xcomp_head" = "head_token_id")) |>
      dplyr::left_join(all_objects, by = c("doc_id", "sentence_id", "xcomp_head" = "obj_head"),
                relationship = "many-to-many") |>
      dplyr::group_by(doc_id, sentence_id, token_id) |>
      dplyr::slice(1) |>
      dplyr::ungroup() |>
      dplyr::mutate(
        lemma_full = tolower(inf_lemma),
        is_future = FALSE,
        has_modal = FALSE,
        is_conditional = FALSE,
        tag = inf_tag,
        subject_phrase = dplyr::case_when(
          !is.na(object_phrase) & tolower(object_phrase) %in% c("them", "him", "her", "it", "you") ~ object_phrase,
          !is.na(main_subject) ~ main_subject,
          !is.na(object_phrase) ~ object_phrase,
          TRUE ~ NA_character_
        ),
        subject_ner_type = dplyr::case_when(
          !is.na(object_phrase) & tolower(object_phrase) %in% c("them", "him", "her", "it", "you") ~ dplyr::coalesce(obj_ner, ""),
          !is.na(main_ner) ~ main_ner,
          TRUE ~ dplyr::coalesce(obj_ner, "")
        )
      ) |>
      dplyr::filter(!is.na(subject_phrase)) |>
      dplyr::select(doc_id, sentence_id, token_id, lemma_full, is_future, has_modal, is_conditional, tag,
             subject_phrase, subject_ner_type)

    utterances <- dplyr::bind_rows(utterances, utterances_from_xcomp) |> dplyr::distinct()
  }

  # --- E. Coding the Verb ---
  scored_utterances <- utterances |>
    dplyr::mutate(lemma_lower = tolower(lemma_full)) |>
    dplyr::mutate(
      opc_score = dplyr::case_when(
        # 0. NEW: "Verb + against" pattern detection (must come first)
        has_against & lemma_lower %in% verbs_punish_with_against ~ -3,
        has_against & lemma_lower %in% verbs_flip_with_against ~ -1,

        # 1. Check Punish (-3) - Future or Conditional Punish -> Threat (-2)
        lemma_lower %in% verbs_punish ~ ifelse(is_future | is_conditional, -2, -3),
        # 2. Check Reward (+3) - Future or Conditional Reward -> Promise (+2)
        lemma_lower %in% verbs_reward ~ ifelse(is_future | is_conditional, 2, 3),
        # 3. Words (Unchanged by future tense, they remain words)
        lemma_lower %in% verbs_threaten ~ -2,
        lemma_lower %in% verbs_promise ~ 2,
        lemma_lower %in% verbs_oppose ~ -1,
        lemma_lower %in% verbs_support ~ 1,

        # 4. Modal constructions
        has_modal & !(lemma_lower %in% c(verbs_punish, verbs_threaten, verbs_oppose)) ~ 1,

        # 5. Nominalizations
        lemma_lower %in% nouns_punish ~ -3,
        lemma_lower %in% nouns_reward ~ 3,
        lemma_lower %in% nouns_threaten ~ -2,
        lemma_lower %in% nouns_oppose ~ -1,
        lemma_lower %in% nouns_support ~ 1,
        lemma_lower %in% nouns_promise ~ 2,

        TRUE ~ NA_real_
      )
    ) |>
    dplyr::filter(!is.na(opc_score))

  # --- F. Specify Subject (Self vs Other) ---
  final_coding <- scored_utterances |>
    dplyr::mutate(
      subject_clean = tolower(subject_phrase),
      subject_no_article = stringr::str_remove(subject_clean, "^(the|a|an)\\s+"),
      ner_type_clean = stringr::str_replace(subject_ner_type, "^[BI]-", "")
    ) |>
    dplyr::mutate(
      # Step 1: Check for Self entities
      is_self = (subject_no_article %in% own_terms | subject_clean %in% own_terms),

      # Step 2: Check for Other entities (exact match with country/org list)
      is_other_exact = subject_no_article %in% other_terms | subject_clean %in% other_terms,

      # Step 3: Check for common other pronouns
      is_other_pronoun = subject_clean %in% c("they", "them", "he", "she", "it", "him", "her", "you"),

      # Step 4: Check for other indicators by name patterns (expanded)
      is_other_pattern = grepl(paste0("(",
        "iraq|iran|soviet|russia|china|north korea|syria|libya|cuba|vietnam|afghanistan|panama|",
        "japan|germany|france|britain|british|uk|england|poland|hungary|czechoslovakia|",
        "israel|egypt|saudi|jordan|lebanon|kuwait|pakistan|india|mexico|canada|brazil|",
        "saddam|hussein|gorbachev|noriega|castro|khomeini|assad|qaddafi|kim|kohl|mitterrand|",
        "thatcher|major|yeltsin|deng|xiaoping|arafat|rabin|mubarak|shamir|",
        "communist|regime|enemy|terrorist|aggressor|dictator|tyrant|invader|occupier|",
        "ally|allies|partner|partners|friend|friends|neighbor|neighbors|neighbour|neighbours|",
        "congress|senate|house|parliament|legislature|court|courts|supreme court|",
        "administration|cabinet|department|agency|agencies|",
        "iraqis|iranians|soviets|russians|chinese|japanese|germans|french|british|",
        "forces|troops|army|military|authorities|officials|leaders|members|delegates|",
        "representatives|senators|congressmen|lawmakers|politicians|diplomats|ambassadors|",
        "un|united nations|nato|opec|kremlin|politburo|ec|european community|",
        "world bank|imf|oas|organization|coalition|alliance|bloc|pact|league|",
        "government|governments|president|prime minister|chancellor|king|queen|",
        "secretary|minister|ministers|spokesman|spokesperson|official|director|chief|",
        "media|press|newspaper|reporter|journalist|critic|critics|opposition|",
        "people|peoples|citizens|nation|nations|society|public|",
        "group|groups|party|parties|faction|movement|rebels|revolutionaries|insurgents",
      ")"), subject_clean, ignore.case = TRUE),

      # Step 5: Check NER entity type
      is_ner_entity = ner_type_clean %in% c("GPE", "ORG", "PERSON", "NORP", "FAC", "LOC"),

      # Step 6: Check for common "Other" agent nouns
      is_other_agent_noun = grepl(paste0("(",
        "authority|authorities|delegation|delegations|leadership|",
        "adversary|adversaries|enemy|enemies|opponent|opponents|",
        "community|communities|population|populations|masses|",
        "world|globe|humanity|mankind|international|foreign|",
        "those|these|others|other|another|",
        "actor|actors|player|players|participant|participants|",
        "side|sides|counterpart|counterparts|",
        "body|bodies|council|councils|committee|committees|",
        "assembly|assemblies|delegation|delegations|mission|missions",
      ")"), subject_clean, ignore.case = TRUE),

      # Step 7: Default
      has_country_word = grepl("(nation|country|state|republic|kingdom|empire|federation|union)",
                               subject_clean, ignore.case = TRUE),

      # Step 8: Detect abstract collective entities
      is_abstract_collective = grepl(paste0("(",
        "all nations|all peoples|all countries|all citizens|all people|",
        "all men|all women|all americans|all humanity|all mankind|",
        "every nation|every people|every country|every citizen|every person|",
        "every man|every woman|every american|",
        "nations of the world|peoples of the world|countries of the world|",
        "people of the world|citizens of the world|",
        "humanity|mankind|humankind|civilized world|free world|",
        "civilized nations|free peoples|democratic nations|",
        "audience|listeners|viewers|readers|voters|electorate|",
        "fellow americans|fellow citizens|countrymen|compatriots",
      ")"), subject_clean, ignore.case = TRUE)
    ) |>
    dplyr::mutate(
      entity_type = dplyr::case_when(
        is_self ~ "Self",
        is_other_exact ~ "Other",
        is_other_pronoun ~ "Other",
        is_other_pattern ~ "Other",
        is_other_agent_noun ~ "Other",
        is_abstract_collective ~ "Other",
        is_ner_entity & !is_self ~ "Other",
        has_country_word & !is_self ~ "Other",
        TRUE ~ NA_character_
      )
    ) |>
    dplyr::filter(!is.na(entity_type)) |>
    dplyr::select(opc_score, entity_type)

  if (nrow(final_coding) == 0) {

    if(!bootstrap) {

      return(tibble::tibble(P1 = 0, P2 = 0, P3 = 0, P4 = 0, P5 = 0,
                    I1 = 0, I2 = 0, I3 = 0, I4a = 0, I4b = 0,
                    I5_punish = 0, I5_threat = 0, I5_oppose = 0,
                    I5_support = 0, I5_promise = 0, I5_reward = 0,
                    self_reward = 0, self_promise = 0, self_support = 0,
                    self_oppose = 0, self_threat = 0, self_punish = 0,
                    other_reward = 0, other_promise = 0, other_support = 0,
                    other_oppose = 0, other_threat = 0, other_punish = 0))

    } else {

      return(tibble::tibble(meanP1 = 0, meanP2 = 0, meanP3 = 0, meanP4 = 0, meanP5 = 0,
                    meanI1 = 0, meanI2 = 0, meanI3 = 0, meanI4a = 0, meanI4b = 0,
                    meanI5_punish = 0, meanI5_threat = 0, meanI5_oppose = 0,
                    meanI5_support = 0, meanI5_promise = 0, meanI5_reward = 0,
                    varP1 = 0, varP2 = 0, varP3 = 0, varP4 = 0, varP5 = 0,
                    varI1 = 0, varI2 = 0, varI3 = 0, varI4a = 0, varI4b = 0,
                    varI5_punish = 0, varI5_threat = 0, varI5_oppose = 0,
                    varI5_support = 0, varI5_promise = 0, varI5_reward = 0,
                    meanself_reward = 0, meanself_promise = 0, meanself_support = 0,
                    meanself_oppose = 0, meanself_threat = 0, meanself_punish = 0,
                    meanother_reward = 0, meanother_promise = 0, meanother_support = 0,
                    meanother_oppose = 0, meanother_threat = 0, meanother_punish = 0,
                    varself_reward = 0, varself_promise = 0, varself_support = 0,
                    varself_oppose = 0, varself_threat = 0, varself_punish = 0,
                    varother_reward = 0, varother_promise = 0, varother_support = 0,
                    varother_oppose = 0, varother_threat = 0, varother_punish = 0,
                    ))

    }

  } else {

    if(!bootstrap) {

      res <- get_scores(final_coding)

      return(res)

    } else {

      boot_counts <- replicate(B, {

        final_coding$rowid <- 1:nrow(final_coding)
        sampled_id <- final_coding$rowid[sample(nrow(final_coding), replace = TRUE)]
        sample_df <- tibble::tibble(rowid = sampled_id)
        final_coding_sampled <- sample_df |>
          dplyr::inner_join(final_coding, by = "rowid", relationship = "many-to-many")

        get_scores(final_coding_sampled)

      }, simplify = TRUE)

      boot_counts_num <- apply(boot_counts, 2, as.numeric)
      rownames(boot_counts_num) <- rownames(boot_counts)
      row_means <- rowMeans(boot_counts_num, na.rm = TRUE)
      row_vars  <- apply(boot_counts_num, 1, stats::var, na.rm = TRUE)

      stats_vec <- c(row_means, row_vars)
      stats_names <- c(paste0("mean", names(row_means)),
                       paste0("var",  names(row_vars)))
      names(stats_vec) <- stats_names

      res <- dplyr::bind_rows(stats_vec)

      return(res)

    }

  }

}

#' International organizations list
#'
#' A character vector of international organization names and acronyms
#' used for entity detection across the package.
#'
#' @keywords internal
io <- c(
  "The United Nations", "UN", "United Nations", "European Union", "EU",
  "NATO", "North Atlantic Treaty Organization",
  "World Health Organization", "WHO",
  "International Monetary Fund", "IMF", "World Bank",
  "World Trade Organization", "WTO", "African Union", "AU",
  "ASEAN", "Association of Southeast Asian Nations",
  "UNESCO", "UNICEF", "FAO", "Food and Agriculture Organization",
  "International Criminal Court", "ICC", "Interpol",
  "UNHCR",
  "Organization of American States", "OAS", "League of Arab States",
  "International Olympic Committee", "IOC")

#' Add dots to acronyms
#'
#' Converts a short uppercase acronym into dotted forms (e.g., "US" becomes
#' "U.S" and "U.S.").
#'
#' @param x A character string.
#' @return A character vector of dotted variants, or \code{character(0)} if
#'   the input is not a 2-4 letter acronym.
#' @keywords internal
acronym_dots <- function(x) {
  x0 <- toupper(x)
  if (!stringr::str_detect(x0, "^[A-Z]{2,4}$")) return(character(0))
  with_dots <- paste(strsplit(x0, "")[[1]], collapse = ".")
  c(with_dots, paste0(with_dots, "."))
}

#' Expand country name aliases
#'
#' Given a country name or code, returns all known aliases including
#' official names, ISO codes, CLDR names, and dotted acronym variants.
#'
#' @param term A character string representing a country name or code.
#' @return A character vector of lowercased country name aliases.
#' @keywords internal
expand_aliases_country <- function(term) {
  canon <- countrycode::countryname(term, "country.name.en", warn = FALSE)
  if (is.na(canon)) {
    return(unique(c(tolower(term), tolower(acronym_dots(term)))))
  }
  row <- countrycode::codelist |> dplyr::filter(country.name.en == canon)
  if (nrow(row) == 0) return(unique(tolower(c(term, canon))))

  raw <- c(
    row$country.name.en,
    row$cldr.name.en,
    row$cldr.short.en,
    row$cldr.variant.en,
    row$un.name.en,
    row$iso.name.en,
    row$iso2c, row$iso3c
  ) |> unlist(use.names = FALSE)

  raw <- raw[!is.na(raw) & raw != ""]
  dotted <- unlist(lapply(raw, acronym_dots))
  tolower(unique(c(raw, dotted)))
}

test_that("exported functions exist", {
  expect_true(is.function(get_oca))
  expect_true(is.function(get_power))
  expect_true(is.function(get_dist))
  expect_true(is.function(get_nat))
  expect_true(is.function(get_aff))
  expect_true(is.function(get_ctrl))
  expect_true(is.function(get_complex))
  expect_true(is.function(get_task))
  expect_true(is.function(get_conf))
  expect_true(is.function(get_lta))
})

test_that("LTA functions have correct parameters", {
  # Functions requiring own_entity
  for (fn_name in c("get_power", "get_dist", "get_nat", "get_aff", "get_ctrl")) {
    fn <- get(fn_name)
    expect_true("own_entity" %in% names(formals(fn)),
                info = paste(fn_name, "should have own_entity param"))
    expect_true("text" %in% names(formals(fn)),
                info = paste(fn_name, "should have text param"))
    expect_true("bootstrap" %in% names(formals(fn)),
                info = paste(fn_name, "should have bootstrap param"))
  }

  # Functions NOT requiring own_entity
  for (fn_name in c("get_complex", "get_task", "get_conf")) {
    fn <- get(fn_name)
    expect_true("text" %in% names(formals(fn)),
                info = paste(fn_name, "should have text param"))
    expect_true("bootstrap" %in% names(formals(fn)),
                info = paste(fn_name, "should have bootstrap param"))
  }

  # OCA function
  expect_true("own_entity" %in% names(formals(get_oca)))
  expect_true("text" %in% names(formals(get_oca)))

  # LTA wrapper
  expect_true("own_entity" %in% names(formals(get_lta)))
  expect_true("text" %in% names(formals(get_lta)))
  expect_true("bootstrap" %in% names(formals(get_lta)))
  expect_true("B" %in% names(formals(get_lta)))
})

---
title: "Operational Code Analysis (OCA)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Operational Code Analysis (OCA)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

This vignette demonstrates Operational Code Analysis (OCA) with leadeR.

## Setup

```{r}
library(leadeR)

spacyr::spacy_initialize()
set.seed(02138)

own_ent <- "United States"
B <- 1000

jfk <- clean_text(jfk19571101)
```

## Computing OCA indices

The `get_oca()` function computes the VICS (Verbs in Context System)
operational code indices from text. It returns the 10 standard indices
(P1--P5 and I1--I5) along with the underlying raw counts.

Set `bootstrap = TRUE` to obtain bootstrap means and variances.

```{r}
res_oca <- get_oca(own_entity = own_ent, text = jfk, bootstrap = TRUE, B = B)
print(res_oca)
```

### Philosophical beliefs (P indices)

| Index | Description                                           |
|-------|-------------------------------------------------------|
| P1    | Nature of the political universe (friendly/hostile)   |
| P2    | Realization of political values (optimism/pessimism)  |
| P3    | Predictability of the political future                |
| P4    | Control over historical development                   |
| P5    | Role of chance                                        |

### Instrumental beliefs (I indices)

| Index | Description                                           |
|-------|-------------------------------------------------------|
| I1    | Strategic approach to goals (cooperative/conflictual) |
| I2    | Intensity of tactics (cooperative/conflictual)        |
| I3    | Risk orientation                                      |
| I4    | Flexibility of tactics (a and b sub-indices)          |
| I5    | Role of chance                                        |

---
title: "Leadership Trait Analysis (LTA)"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Leadership Trait Analysis (LTA)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

This vignette walks through Leadership Trait Analysis (LTA) with leadeR,
from computing trait scores to classifying leaders into Hermann's typology.

## Setup

```{r}
library(leadeR)
library(data.table)

spacyr::spacy_initialize()
set.seed(02138)

own_ent <- "United States"
B <- 1000

jfk1 <- clean_text(jfk19610120)
jfk2 <- clean_text(jfk19610925)
jfk3 <- clean_text(jfk19630610)
```

## Computing trait scores

### All traits at once

`get_lta()` computes all eight LTA traits in one call and returns a
one-row tibble per speech. Set `bootstrap = TRUE` to obtain bootstrap
means and delta-method variances for the trait proportions.

```{r}
res_lta <- data.table::rbindlist(
  lapply(c(jfk1, jfk2, jfk3), function(x)
    get_lta(own_entity = own_ent, text = x, bootstrap = TRUE, B = B))
)

print(res_lta)
```

The output includes:

- **Bootstrap means and variances** for raw counts (e.g., `meanP`, `varP`,
  `meanOP`, `varOP`)
- **Trait proportions and their delta-method variances** (e.g., `Pp`,
  `varPp`, `D`, `varD`, `C`, `varC`, `Ta`, `varTa`, `Ss`, `varSs`, `Na`,
  `varNa`, `B`, `varB`)

The seven trait proportions are:

| Abbreviation | Trait                              |
|--------------|------------------------------------|
| `Pp`         | Need for power                     |
| `B`          | Belief in ability to control events|
| `C`          | Conceptual complexity              |
| `Ss`         | Self-confidence                    |
| `Ta`         | Task orientation                   |
| `D`          | Distrust                           |
| `Na`         | Nationalism (in-group bias)        |

### Individual traits

You can also compute traits one at a time:

```{r}
res_nat     <- get_nat(own_entity = own_ent, text = jfk1, bootstrap = TRUE, B = B)
res_ctrl    <- get_ctrl(own_entity = own_ent, text = jfk1, bootstrap = TRUE, B = B)
res_power   <- get_power(own_entity = own_ent, text = jfk1, bootstrap = TRUE, B = B)
res_aff     <- get_aff(own_entity = own_ent, text = jfk1, bootstrap = TRUE, B = B)
res_dist    <- get_dist(own_entity = own_ent, text = jfk1, bootstrap = TRUE, B = B)
res_complex <- get_complex(text = jfk1, bootstrap = TRUE, B = B)
res_conf    <- get_conf(text = jfk1, bootstrap = TRUE, B = B)
res_task    <- get_task(text = jfk1, bootstrap = TRUE, B = B)
```

## Leader typology

Using the per-speech LTA traits, `type_lta()` aggregates scores across
speeches and classifies the leader into Hermann's typology.

### Aggregation methods

The way speech-level traits are aggregated matters for classification.
`type_lta()` supports two approaches:

**Simple mean** (`precision_weighted = FALSE`, the default):
Takes the arithmetic mean of each trait across speeches. All speeches
contribute equally regardless of how precisely each trait was estimated.

**Precision-weighted mean** (`precision_weighted = TRUE`):
Uses inverse-variance weighting via random-effects meta-analysis
(`metafor::rma()` with REML). Speeches with lower bootstrap variance
(i.e., more precisely estimated traits) receive higher weight. This
requires that `get_lta()` was run with `bootstrap = TRUE` so that the
variance columns (`varPp`, `varB`, etc.) are available.

```{r}
# Simple mean
type_lta(res_lta)

# Precision-weighted
type_lta(res_lta, precision_weighted = TRUE)
```

### Classification dimensions

`type_lta()` classifies the leader along three dimensions:

**Constraint** (Respect vs. Challenge): Based on need for power (`Pp`)
and belief in control (`B`). A leader who scores below both thresholds
respects constraints; otherwise, they challenge constraints.

**Openness** (Open vs. Closed): Based on conceptual complexity (`C`) and
self-confidence (`Ss`). A leader is open to information if complexity
exceeds self-confidence, or if both exceed their respective high
thresholds.

**Motivation toward world** (four categories): Based on distrust (`D`) and
nationalism (`Na`):

| Distrust | Nationalism | Motivation                      |
|----------|-------------|---------------------------------|
| Low      | Low         | Cooperative                     |
| Low      | High        | Cooperative (in-group bias)     |
| High     | Low         | Competitive (out-group focus)   |
| High     | High        | Competitive                     |

### Leadership styles

Constraint, openness, and task orientation (`Ta`) map to one of eight
leadership styles:

| Constraint | Openness | Task orientation | Style           |
|------------|----------|------------------|-----------------|
| Challenge  | Closed   | Problem          | Expansionistic  |
| Challenge  | Closed   | Relationship     | Evangelistic    |
| Challenge  | Open     | Problem          | Incremental     |
| Challenge  | Open     | Relationship     | Charismatic     |
| Respect    | Closed   | Problem          | Directive       |
| Respect    | Closed   | Relationship     | Consultative    |
| Respect    | Open     | Problem          | Reactive        |
| Respect    | Open     | Relationship     | Accommodative   |

```{r}
out_lta <- type_lta(res_lta, precision_weighted = TRUE)
out_lta[, c("constraint", "openness", "motivation_toward_world", "typology")]
```

### Custom thresholds

All classification thresholds are configurable. The defaults correspond to
the norming sample:

```{r}
type_lta(
  res_lta,
  precision_weighted = TRUE,
  need_for_power  = 0.50,
  control         = 0.44,
  complex_high    = 0.56,
  confidence_high = 0.81,
  task            = 0.59,
  distrust        = 0.41,
  ingroup         = 0.42
)
```

---
title: "Installation, Initialization, and Data Cleaning"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Installation, Initialization, and Data Cleaning}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

## Prerequisites

leadeR relies on [spaCy](https://spacy.io/), a Python NLP library, via the
[spacyr](https://spacyr.quanteda.io/) R package. You will need:

- **Python** (3.8 or later)
- **spaCy** with an English language model

Install spaCy and the English model from a terminal:

```bash
pip install spacy
python -m spacy download en_core_web_sm
```

## Installing leadeR

Install leadeR from GitHub:

```{r}
# install.packages("remotes")
remotes::install_github("mmukaigawara/leadeR")
```

## Initialization

Before using any leadeR function, initialize spaCy and (optionally) set a
seed for reproducibility of bootstrap results.

```{r}
library(leadeR)
library(data.table)

spacyr::spacy_initialize()

set.seed(02138)
```

## Sample data

The package ships with three speeches by John F. Kennedy:

| Dataset          | Date               | Occasion                                    |
|------------------|--------------------|---------------------------------------------|
| `jfk19610120`    | January 20, 1961   | Inaugural Address                           |
| `jfk19610925`    | September 25, 1961 | Address Before the UN General Assembly      |
| `jfk19630610`    | June 10, 1963      | Commencement Address at American University |

```{r}
head(jfk19571101)
```

## Text cleaning

Speech transcripts often contain editorial annotations in brackets,
parentheses, or curly braces. The `clean_text()` function removes these
and normalizes whitespace.

```{r}
jfk1 <- clean_text(jfk19610120)
jfk2 <- clean_text(jfk19610925)
jfk3 <- clean_text(jfk19630610)
```

Users may need additional cleaning steps depending on the source of their
text data (e.g., removing headers, footers, or speaker labels).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/self_confidence.R
\name{get_conf}
\alias{get_conf}
\title{Compute self-confidence scores}
\usage{
get_conf(text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}
}
\value{
A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
columns are \code{SC} and \code{OSC}. When \code{bootstrap = TRUE}, columns
are \code{meanSC}, \code{meanOSC}, \code{varSC}, \code{varOSC}.
}
\description{
Classifies first-person pronoun occurrences in the text as self-confident
(SC) or other-self-confident (OSC) based on sentence-level context patterns
covering three conditions: self as instigator, self as authority, and self
as recipient of positive recognition.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lta.R
\name{get_lta}
\alias{get_lta}
\title{Leadership Trait Analysis (LTA)}
\usage{
get_lta(own_entity, text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{own_entity}{A character string identifying the speaker's country or
entity (e.g., \code{"United States"}).}

\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical. If \code{TRUE}, returns bootstrap means and variances
instead of raw counts. Default is \code{FALSE}.}

\item{B}{Number of bootstrap iterations. Default is \code{1000}.}
}
\value{
A one-row \link[tibble:tibble]{tibble::tibble}.

When \code{bootstrap = FALSE}, columns include raw counts (\code{P}, \code{OP}, \code{A},
\code{OA}, \code{S}, \code{OS}, \code{HC}, \code{LC}, \code{TI}, \code{IP}, \code{SC}, \code{OSC}, \code{N}, \code{ON}, \code{IC},
\code{OC}) plus trait proportions (\code{Pp}, \code{D}, \code{C}, \code{Ta}, \code{Ss}, \code{Na}, \code{B}).

When \code{bootstrap = TRUE}, columns include bootstrap means and variances
for the raw counts (\code{meanP}, \code{varP}, \code{meanOP}, \code{varOP}, ...) plus trait
proportions and their delta-method variances (\code{Pp}, \code{varPp}, \code{D}, \code{varD},
\code{C}, \code{varC}, \code{Ta}, \code{varTa}, \code{Ss}, \code{varSs}, \code{Na}, \code{varNa}, \code{B}, \code{varB}).
}
\description{
Runs all eight LTA trait functions at once and returns a single combined
tibble. The eight traits are: power, affiliation, distrust, conceptual
complexity, task orientation, self-confidence, nationalism, and control.
}
\examples{
\donttest{
# Requires spaCy to be installed; see spacyr::spacy_install().
spacyr::spacy_initialize()
get_lta(own_entity = "United States", text = "We will defend our nation.")
get_lta(own_entity = "United States", text = "We will defend our nation.",
        bootstrap = TRUE, B = 500)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand_aliases.R
\name{acronym_dots}
\alias{acronym_dots}
\title{Add dots to acronyms}
\usage{
acronym_dots(x)
}
\arguments{
\item{x}{A character string.}
}
\value{
A character vector of dotted variants, or \code{character(0)} if
the input is not a 2-4 letter acronym.
}
\description{
Converts a short uppercase acronym into dotted forms (e.g., "US" becomes
"U.S" and "U.S.").
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distrust.R
\name{get_dist}
\alias{get_dist}
\title{Compute distrust scores}
\usage{
get_dist(own_entity, text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{own_entity}{A character vector of entity names representing the
speaker's own country or group.}

\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}
}
\value{
A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
columns are \code{S} and \code{OS}. When \code{bootstrap = TRUE}, columns
are \code{meanS}, \code{varS}, \code{meanOS}, \code{varOS}.
}
\description{
Performs per-entity classification of references in the text as
suspicious (S) or other-suspicious (OS) based on distrust verbs,
harmful nominals, and contextual modifiers.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{strip_quoted}
\alias{strip_quoted}
\title{Strip quoted text from a string}
\usage{
strip_quoted(x)
}
\arguments{
\item{x}{A character string.}
}
\value{
The string with quoted passages removed.
}
\description{
Strip quoted text from a string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{prep_phrase_regex}
\alias{prep_phrase_regex}
\title{Prepare phrase regex pattern}
\usage{
prep_phrase_regex(phrases)
}
\arguments{
\item{phrases}{Character vector of phrases.}
}
\value{
A single regex string.
}
\description{
Prepare phrase regex pattern
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leadeR-package.R
\docType{package}
\name{leadeR-package}
\alias{leadeR}
\alias{leadeR-package}
\title{leadeR: Profiling Leaders at a Distance}
\description{
The leadeR package profiles political leaders using text analysis,
implementing Leadership Trait Analysis (LTA) and Operational Code
Analysis (OCA). You provide text data and the package performs the
analyses.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mmukaigawara/leadeR}
}

}
\author{
\strong{Maintainer}: Mitsuru Mukaigawara \email{mitsuru_mukaigawara@g.harvard.edu} (\href{https://orcid.org/0000-0001-6530-2083}{ORCID})

Authors:
\itemize{
  \item Mitsuru Mukaigawara \email{mitsuru_mukaigawara@g.harvard.edu} (\href{https://orcid.org/0000-0001-6530-2083}{ORCID})
  \item Joshua Kertzer \email{jkertzer@gov.harvard.edu} (\href{https://orcid.org/0000-0002-7358-0638}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{prep_wordstem_regex}
\alias{prep_wordstem_regex}
\title{Prepare word-stem regex pattern}
\usage{
prep_wordstem_regex(stems)
}
\arguments{
\item{stems}{Character vector of word stems.}
}
\value{
A single regex string.
}
\description{
Prepare word-stem regex pattern
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task.R
\name{get_task}
\alias{get_task}
\title{Compute task orientation scores}
\usage{
get_task(text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}
}
\value{
A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
columns are \code{TI} and \code{IP}. When \code{bootstrap = TRUE}, columns
are \code{meanTI}, \code{meanIP}, \code{varTI}, \code{varIP}.
}
\description{
Classifies lemmas in the parsed text as task-instrumental (TI) or
interpersonal (IP) based on codebook word lists, with token-level
quote and negation handling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean_text.R
\name{clean_text}
\alias{clean_text}
\title{Clean text for analysis}
\usage{
clean_text(x)
}
\arguments{
\item{x}{A character vector of text to clean.}
}
\value{
A character vector with annotations removed and whitespace normalized.
}
\description{
Removes editorial annotations in square brackets, parentheses, and
curly braces from political speech transcripts. Also normalizes
whitespace and collapses double em-dashes left behind by removals.
}
\examples{
clean_text("We must act now [applause] for the future.")
clean_text("The president (speaking loudly) left the room.")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type_lta.R
\name{type_lta}
\alias{type_lta}
\title{Classify LTA Traits into Hermann's Leadership Typology}
\usage{
type_lta(
  lta,
  precision_weighted = FALSE,
  need_for_power = 0.5,
  control = 0.44,
  complex_high = 0.56,
  confidence_high = 0.81,
  task = 0.59,
  distrust = 0.41,
  ingroup = 0.42
)
}
\arguments{
\item{lta}{A data frame with one row per speech, as returned by
\code{\link[=get_lta]{get_lta()}}. Must contain columns \code{Pp}, \code{B}, \code{C}, \code{Ss}, \code{Ta}, \code{D}, \code{Na}.
When \code{precision_weighted = TRUE}, must also contain \code{varPp}, \code{varB},
\code{varC}, \code{varSs}, \code{varTa}, \code{varD}, \code{varNa}.}

\item{precision_weighted}{Logical. If \code{FALSE} (default), aggregate traits
with a simple mean. If \code{TRUE}, use inverse-variance (precision) weighting
via random-effects meta-analysis (\code{metafor::rma()}).}

\item{need_for_power}{Threshold for the need-for-power trait (\code{Pp}).
Default \code{0.50}.}

\item{control}{Threshold for belief in ability to control events (\code{B}).
Default \code{0.44}.}

\item{complex_high}{Threshold for high conceptual complexity (\code{C}).
Default \code{0.56}.}

\item{confidence_high}{Threshold for high self-confidence (\code{Ss}).
Default \code{0.81}.}

\item{task}{Threshold for task orientation (\code{Ta}). Default \code{0.59}.}

\item{distrust}{Threshold for distrust (\code{D}). Default \code{0.41}.}

\item{ingroup}{Threshold for in-group bias / nationalism (\code{Na}).
Default \code{0.42}.}
}
\value{
A one-row \link[tibble:tibble]{tibble::tibble} with aggregated trait values, standard
errors (when \code{precision_weighted = TRUE}), and classification columns:
\code{constraint}, \code{openness}, \code{motivation_toward_world}, \code{task_orientation},
\code{typology}, and \code{method}.
}
\description{
Takes per-speech LTA output (from \code{\link[=get_lta]{get_lta()}}) and aggregates trait scores
across speeches, then classifies the leader along three dimensions
(constraint, openness, motivation toward world) and maps the first two plus
task orientation to one of eight leadership styles.
}
\examples{
# Simple-mean aggregation works on any data frame with the seven trait
# columns; here we use a small illustrative input.
example_lta <- data.frame(
  Pp = c(0.45, 0.52, 0.48),
  B  = c(0.40, 0.48, 0.43),
  C  = c(0.60, 0.55, 0.58),
  Ss = c(0.75, 0.82, 0.79),
  Ta = c(0.62, 0.58, 0.65),
  D  = c(0.35, 0.42, 0.38),
  Na = c(0.38, 0.45, 0.41)
)
type_lta(example_lta)

\donttest{
# Precision-weighted aggregation needs per-trait variances, which come
# from get_lta(..., bootstrap = TRUE). Requires spaCy to be installed.
spacyr::spacy_initialize()
res <- data.table::rbindlist(
  lapply(c(jfk19610120, jfk19610925, jfk19630610), function(x)
    get_lta(own_entity = "United States", text = clean_text(x),
            bootstrap = TRUE, B = 1000))
)
type_lta(res, precision_weighted = TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{jfk19630610}
\alias{jfk19630610}
\title{JFK Speech: Commencement Address at American University (June 10, 1963)}
\format{
A character string containing the full text of the speech.
}
\source{
\url{https://www.govinfo.gov/app/details/PPP-1963-book1}.
These original source texts are U.S. government works and are not subject to
copyright protection in the United States under 17 U.S.C. § 105.
The package authors do not claim copyright in the original presidential speech texts.
}
\usage{
jfk19630610
}
\description{
Full text of Senator John F. Kennedy's commencement address at American University on June 10, 1963.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/control.R
\name{get_ctrl}
\alias{get_ctrl}
\title{Compute control (need for power) scores}
\usage{
get_ctrl(own_entity, text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{own_entity}{A character vector of entity names representing the
speaker's own country or group.}

\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}
}
\value{
A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
columns are \code{IC} and \code{OC}. When \code{bootstrap = TRUE}, columns
are \code{meanIC}, \code{meanOC}, \code{varIC}, \code{varOC}.
}
\description{
Classifies verbs associated with first-person subjects as
instrumental-control (IC) or other-control (OC) using dependency parsing
to link subjects to their governing verbs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-entities.R
\name{build_country_list}
\alias{build_country_list}
\title{Build country list for entity matching}
\usage{
build_country_list()
}
\value{
A character vector of country names.
}
\description{
Constructs a vector of lowercased country names including aliases,
without international organizations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand_aliases.R
\name{io}
\alias{io}
\title{International organizations list}
\usage{
io
}
\description{
A character vector of international organization names and acronyms
used for entity detection across the package.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand_aliases.R
\name{expand_aliases_country}
\alias{expand_aliases_country}
\title{Expand country name aliases}
\usage{
expand_aliases_country(term)
}
\arguments{
\item{term}{A character string representing a country name or code.}
}
\value{
A character vector of lowercased country name aliases.
}
\description{
Given a country name or code, returns all known aliases including
official names, ISO codes, CLDR names, and dotted acronym variants.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opcode.R
\name{get_oca}
\alias{get_oca}
\title{Compute operational code (VICS) scores}
\usage{
get_oca(own_entity, text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{own_entity}{A character vector of entity names representing the
speaker's own country or group.}

\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates for all indices. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}
}
\value{
A one-row \code{\link[tibble]{tibble}} with P1--P5, I1--I5, and
raw Self/Other counts by score category.
}
\description{
Performs full VICS (Verbs in Context System) operational code analysis on
a speech text, classifying verb-entity pairs as Self/Other and scoring
them on a -3 to +3 scale (Punish to Reward). Returns P1--P5 and I1--I5
indices.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opcode.R
\name{get_scores}
\alias{get_scores}
\title{Compute VICS operational code scores from verb-entity coding}
\usage{
get_scores(dat)
}
\arguments{
\item{dat}{A data frame with columns \code{opc_score} (integer, -3 to 3)
and \code{entity_type} (character, "Self" or "Other").}
}
\value{
A one-row \code{\link[tibble]{tibble}} with P1--P5, I1--I5, and
raw counts.
}
\description{
Internal function that takes a data frame of coded utterances (with
\code{opc_score} and \code{entity_type} columns) and computes the full
set of P and I indices.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{prep_exact_regex}
\alias{prep_exact_regex}
\title{Prepare exact-word regex pattern}
\usage{
prep_exact_regex(words)
}
\arguments{
\item{words}{Character vector of words to match exactly.}
}
\value{
A single regex string.
}
\description{
Prepare exact-word regex pattern
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{count_valid}
\alias{count_valid}
\title{Count valid (non-negated) pattern matches}
\usage{
count_valid(txt, pattern, window_chars = 40)
}
\arguments{
\item{txt}{A character string to search.}

\item{pattern}{A regex pattern to match.}

\item{window_chars}{Integer; number of characters to look back for negation.}
}
\value{
Integer count of valid matches.
}
\description{
Count valid (non-negated) pattern matches
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-entities.R
\name{build_entities_corpus}
\alias{build_entities_corpus}
\title{Build entities corpus for entity matching}
\usage{
build_entities_corpus()
}
\value{
A character vector of entity names.
}
\description{
Constructs a vector of lowercased international organization names and
country names (including aliases) for use in entity detection.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{prep_lc_regex}
\alias{prep_lc_regex}
\title{Prepare low-complexity regex pattern}
\usage{
prep_lc_regex(items)
}
\arguments{
\item{items}{Character vector of low-complexity words and phrases.}
}
\value{
A single regex string.
}
\description{
Prepare low-complexity regex pattern
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/power.R
\name{get_power}
\alias{get_power}
\title{Compute Power Score (LTA)}
\usage{
get_power(own_entity, text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{own_entity}{Character vector of the speaker's country/entity name(s).}

\item{text}{Character string of the speech text to analyse.}

\item{bootstrap}{Logical; if TRUE, return bootstrap means and variances.}

\item{B}{Number of bootstrap replications (default 1000).}
}
\value{
A tibble with columns P and OP (or meanP, meanOP, varP, varOP if
bootstrap = TRUE).
}
\description{
Classifies verbs in text as reflecting Power (P) or Other Power (OP)
based on the Leadership Trait Analysis codebook.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complexity.R
\name{get_complex}
\alias{get_complex}
\title{Compute conceptual complexity scores}
\usage{
get_complex(
  text,
  bootstrap = FALSE,
  B = 1000,
  quote_strip = TRUE,
  window_chars = 40
)
}
\arguments{
\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}

\item{quote_strip}{Logical; if \code{TRUE}, remove quoted text before
counting. Default is \code{TRUE}.}

\item{window_chars}{Integer; number of characters to look back for
negation context. Default is 40.}
}
\value{
A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
columns are \code{HC} and \code{LC}. When \code{bootstrap = TRUE}, columns
are \code{meanHC}, \code{varHC}, \code{meanLC}, \code{varLC}.
}
\description{
Counts high-complexity (HC) and low-complexity (LC) markers in the text,
using word stems, phrases, and exact words with negation-aware counting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/affiliation.R
\name{get_aff}
\alias{get_aff}
\title{Compute affiliation scores}
\usage{
get_aff(own_entity, text, bootstrap = FALSE, B = 1000)
}
\arguments{
\item{own_entity}{A character vector of entity names representing the
speaker's own country or group.}

\item{text}{A character string containing the speech text to analyse.}

\item{bootstrap}{Logical; if \code{TRUE}, return bootstrapped mean and
variance estimates. Default is \code{FALSE}.}

\item{B}{Integer; number of bootstrap replicates. Default is 1000.}
}
\value{
A one-row \code{\link[tibble]{tibble}}. When \code{bootstrap = FALSE},
columns are \code{A} and \code{OA}. When \code{bootstrap = TRUE}, columns
are \code{meanA}, \code{meanOA}, \code{varA}, \code{varOA}.
}
\description{
Measures the affiliation orientation of a leader's speech by classifying
verb-level actions as affiliative (A) or other-affiliative (OA) based on
the speaker's own-entity references and sentence-level context.
}
